MODULE_NAME='M_Genelec_44x0A_COMM, REV04' (DEV vdvSpkr__1, 
					   DEV dvSpkr___1, 
					   CHAR cIpAddress[])





(******************************************************************************************)
(* Start  History                                                                         *)
(******************************************************************************************)
(*                                                                                        *)
(* 17.09.2020/CL: Rev01 created                                                           *)
(* 25.09.2020/CL: Rev02 created                                                           *)
(* 05.10.2020/CL: Rev03 created                                                           *)
(*                testet with Genelec Model 4420, Software Rev 44x0-1.1.11-202007021238   *)
(* 25.02.2022/CL: Rev04 created                                                           *)
(*                testet with Genelec Model 4420, Software Rev 44x0-1.2.10-202201180940   *)
(*                Implementation of new commands in smart_ip_api_v1_rev_0_7               *)
(*                                                                                        *)
(*                                                                                        *)
(******************************************************************************************)



(*******************************************************************)
(* Important information:                                          *)
(* ----------------------                                          *)
(* Running Genelec's Smart IP Manager Software                     *)
(*                       AND                                       *)
(* connecting to Speaker over API  (e.g. by this AMX Comm module)  *)
(* at the same time is   NOT   supported .                         *)
(*******************************************************************)



(***********************************************************************************)
(* Interface                                                                       *)
(***********************************************************************************)
(* UI/Main to COMM:                                                                *)
(* ================                                                                *)
(*                                                                                 *)
(* SEND_COMMAND:                                                                   *)
(* -------------                                                                   *)
(* VOL_LVL-<level>            <level> = -200..0                                    *)
(* VOL_MUTE-<state>           <state> = TOGGLE | ON | OFF                          *)
(* PRESET-<presetnr>          <presetnr> = 0..5                                    *)
(* POWER-<state>              <state> = STANDBY | ACTIVE | BOOT                    *)
(*                                                                                 *)
(* SET_VOLUME_STEPSIZE-<small>,<large>     Volume step size in 0.1 dB              *)
(*                            Default:  <small>=2dB, <large>=6dB                   *)
(*                            <small> = for small steps (e.g BUTTON_EVENT,PUSH:)   *)
(*                            <large> = for large steps (e.g BUTTON_EVENT,HOLD:)   *)
(*                                                                                 *)
(* SET_VOLUME_RANGE-<min>,<max>            Volume range in 0.1 dB                  *)
(*                            Default:  <min>=-80dB, <max>=0dB                     *)
(*                            <min> = -200..0   (must be < max)                    *)
(*                            <max> = -200..0   (must be > min)                    *)
(*                                                                                 *)
(* GET_DEVICE_INFO-<path>     Get infos about the speaker                          *)
(*                            <path> =  /device/pwr                                *)
(*                                      /audio/inputs                              *)
(*                                      /audio/volume                              *)
(*                                      /events                                    *)
(*                                      /network/zone                              *)
(*                                      /profile/list                              *)
(*                                      /device/id                                 *)
(*                                      /device/info                               *)
(*                                                                                 *)
(*                                                                                 *)
(* CHANNEL:                                                                        *)
(* --------                                                                        *)
(* Channel 24  = VOL_UP          small volume step    (e.g BUTTON_EVENT,PUSH:)     *) 
(* Channel 25  = VOL_DN          small volume step    (e.g BUTTON_EVENT,PUSH:)     *) 
(* Channel 30  = SOURCE_TV1      on/off Toggle for Analog input                    *)
(* Channel 31  = SOURCE_VIDEO1   on/off Toggle for AoIP input channel 1            *)
(* Channel 32  = SOURCE_VIDEO2   on/off Toggle for AoIP input channel 2            *)
(* Channel 140 = GAIN_UP         large volume step    (e.g BUTTON_EVENT,HOLD:)     *) 
(* Channel 141 = GAIN_DN         large volume step    (e.g BUTTON_EVENT,HOLD:)     *) 
(*                                                                                 *)
(*                                                                                 *)
(*                                                                                 *)
(*                                                                                 *)
(* COMM to UI/Main:                                                                *)
(* ================                                                                *)
(*                                                                                 *)
(* SEND_STRING:                                                                    *)
(* ------------                                                                    *)
(* MSG_FROM_MODULE-ERROR,.......                                                   *)
(* DEVICE_INFO-<path>,<property name>,<property value>                             *)
(*                                                                                 *)
(*                                                                                 *)
(* CHANNEL:                                                                        *)
(* --------                                                                        *)
(* Channel 70   = Feedback for Analog input                                        *)
(* Channel 71   = Feedback for AoIP input channel 1                                *)
(* Channel 72   = Feedback for AoIP input channel 2                                *)
(* Channel 199  = VOL_MUTE_FB                                                      *)
(* Channel 251  = DEVICE_COMMUNICATING                                             *)
(* Channel 255  = POWER_FB                                                         *)
(*                                                                                 *)
(*                                                                                 *)
(* LEVEL:                                                                          *)
(* ------                                                                          *)
(* Level 1      = VOL_LVL    Volume level -2000..0     = -200dB..0dB, 0.1dB steps  *)
(*                                                                                 *)
(***********************************************************************************)






// INCLUDE's:
#INCLUDE 'SNAPI.axi'




(***********************************************************)
(*               CONSTANT DEFINITIONS GO BELOW             *)
(***********************************************************)
DEFINE_CONSTANT




// Timeline Identifier:
TL1_ID_FEEDBACK    = 1          // Identifier for Timeline                 
TL2_ID_TIMER1SEC   = 2          // Identifier for Timeline 
TL4_ID_TIMEOUT     = 4          // Identifier for Timeline 
TL5_ID_POLLING     = 5          // Identifier for Timeline 



INTEGER SPKR_RX_BUFFER_SIZE       = 5000
INTEGER SPKR_TX_BUFFER_SIZE       = 1500
CHAR    CrLf[]                    = {$0D,$0A}




(***********************************************************)
(*              STRUCTURE DEFINITIONS GO BELOW             *)
(***********************************************************)
DEFINE_TYPE

STRUCTURE _Authorization
  {
  CHAR    cType[20]	         // <type>
  CHAR    cCredentials[32]	 // <credentials>
  }


  
STRUCTURE _SpkrStatus
  {
  INTEGER nIsOnline	                 // 0=DATA_EVENT,OFFLINE:, 
                                         // 1=DATA_EVENT,ONLINE:
  INTEGER nDEVICE_COMMUNICATING	         // = SNAPI Channel 251 DEVICE_COMMUNICATING
                                         // 0=no RX Data for 12s 
                                         // 1=rx data received   
  INTEGER nPOWER_FB	                 // = SNAPI Channel 255 POWER_FB
                                         // 0=standby 
                                         // 1=active  
  CHAR    cDevice_pwr_State[12]		 // device/pwr property     
                                         // STANDBY, ACTIVE, ISS_SLEEP, PWR_FAIL, (and undocumented: WAIT_STANDBY, WAKEUP)
  FLOAT   fDevice_pwr_PoeAllocatedPwr    // device/pwr property
                                         // Power allocated by PoE PSE (switch) [0.1W].
  INTEGER nDevice_pwr_PoePd15W           // device/pwr property
                         	         // 1=PoE PD (loudspeaker) limits current consumption to 15W
					 // 0=full power is needed (30W)
  CHAR    cMute[5]                       // 'false'   (= unmute)
                                         // 'true'    (= mute)					 
  FLOAT   fVolume                        // -200..0   in 0.1dB steps				 
  INTEGER nInput_Analog                  // 1= analog input is selected				 
  INTEGER nInput_AoIP01                  // 1= AudioOverIP input 1 is selected				 
  INTEGER nInput_AoIP02                  // 1= AudioOverIP input 2 is selected	
  //
  // status variables below just for information:
  FLOAT   fEvents_bsLevel                // events/  property  (Bass output level)
  FLOAT   fEvents_twLevel                // events/  property  (Tweeter output level)
  FLOAT   fEvents_inLevel                // events/  property  (input level)
  FLOAT   fEvents_cpuT                   // events/  property  (CPU temperature)
  FLOAT   fEvents_twCoilT                // events/  property  (Tweeter temperature)
  FLOAT   fEvents_nwInKbps               // events/  property  (Network traffic to host CPU [kbps])
  FLOAT   fEvents_cpuLoad                // events/  property  (CPU load)
  CHAR    cEvents_uptime[32]             // events/  property  (Time from startup)
  CHAR    cDevice_id_barcode[20]         // device/id/  property			 
  CHAR    cDevice_id_mac[20]             // device/id/  property			 
  CHAR    cDevice_id_hwId[32]            // device/id/  property			 
  CHAR    cDevice_id_model[32]           // device/id/  property			 
  CHAR    cDevice_id_modId[32]           // device/id/  property			 
  CHAR    cDevice_id_calibr[32]          // device/id/  property			 
  CHAR    cDevice_info_fwId[64]          // device/info/  property			 
  CHAR    cDevice_info_build[32]         // device/info/  property			 
  CHAR    cDevice_info_baseId[32]        // device/info/  property			 
  CHAR    cDevice_info_hwId[32]          // device/info/  property			 
  CHAR    cDevice_info_model[32]         // device/info/  property			 
  CHAR    cDevice_info_category[32]      // device/info/  property			 
  CHAR    cDevice_info_technology[32]    // device/info/  property			 
  CHAR    cDevice_info_upgradeId[16]     // device/info/  property	
  CHAR    cDevice_info_apiVer[16]        // device/info/  property			 
  INTEGER nDevice_info_confirmFwUpdate   // device/info/  property	
  INTEGER nNetwork_zone_zone             // network/zone/  property	
  CHAR    cNetwork_zone_name[50]         // network/zone/  property
  INTEGER nProfile_list_selected         // profile/list/  property	
  INTEGER nProfile_list_startup          // profile/list/  property	
  CHAR    cProfile_list_name[6][50]      // profile/list/  property	
  }





(***********************************************************)
(*               VARIABLE DEFINITIONS GO BELOW             *)
(***********************************************************)
DEFINE_VARIABLE


// Timelines Interval:
VOLATILE LONG TL1_INTERVAL_FEEDBACK[]            =   {100}                              // 0.1 sec     Interval for timeline
VOLATILE LONG TL2_INTERVAL_TIMER1SEC[]           =   {1000}                             // 1 sec       Interval for timeline
VOLATILE LONG TL4_INTERVAL_TIMEOUT[]             =   {10000}                            // 10 sec      Interval for timeline

VOLATILE INTEGER nSPKR_IP_PORT  = 9000                         // TCP-IP port

VOLATILE INTEGER nSPKR_send_ready                              // 0= sending data to speaker is paused
                                                               // 1= sending data to speaker is allowed
VOLATILE CHAR    cSPKR_RX_BUFFER[SPKR_RX_BUFFER_SIZE]          // buffer for received messages
VOLATILE CHAR    cSPKR_TX_BUFFER[SPKR_TX_BUFFER_SIZE]          // buffer for messages to send


VOLATILE INTEGER nSPKR_DEBUG = 0                               // 0= normal, 1=debug output  to Diagnostics  
                                                               // Note: AMX_LOG must be set to AMX_DEBUG to see debug output in Diagnostics
							       
VOLATILE CHAR    cLast_TX[300]                                 // for debugging
VOLATILE INTEGER nTxCounter                                    // for debugging


VOLATILE INTEGER nPollingPause = 0                             // 0= polling is activ,  >=1 polling is pausing (e.g. for debugging)

VOLATILE _Authorization sAuthorization



VOLATILE _SpkrStatus sSpkrStatus                             // Status of the speaker


// Volume setup:
VOLATILE FLOAT   fVolStepSmall = 2                  // Step size in dB for small steps (e.g. BUTTON_EVENT,PUSH:)
VOLATILE FLOAT   fVolStepLarge = 6                  // Step size in dB for large steps (e.g. BUTTON_EVENT,HOLD:)
VOLATILE FLOAT   fVolMin = -80                      // -200..0  (must be < fVolMax)
VOLATILE FLOAT   fVolMax = 0                        // -200..0  (must be > fVolMin)

VOLATILE CHAR cSET_INPUT_SELECTOR_COMMAND[30]       // temporary char for sending an input selector command in a wait,  e.g.  "A","AoIP01","AoIP02"






(***********************************************************)
(*        SUBROUTINE/FUNCTION DEFINITIONS GO BELOW         *)
(***********************************************************)


(************************************************************************)
(* START  DEFINE_FUNCTION  Send_GET_method  			        *)
(************************************************************************)
DEFINE_FUNCTION Send_GET_method(CHAR cURI[])
  {
  LOCAL_VAR CHAR cGET_Cmd[300]
  
  cGET_Cmd = "'GET ', cURI,' HTTP/1.1', CrLf,
              'Accept: application/json', CrLf,
	      'Connection: keep-alive', CrLf,
	      'Authorization: ',sAuthorization.cType,' ',sAuthorization.cCredentials, CrLf,
	      'Content-Length: 0',CrLf,
	      'Host: ',cIpAddress,':',ITOA(nSPKR_IP_PORT),CrLf,
	       CrLf" 
	       
  WriteToTxBuffer(cGET_Cmd)
  
  // STRING_TO_DIAGNOSTICS("'SPKR_DEBUG: Send_GET_method: ',cURI", 12, __LINE__)
  }
(************************************************************************)
(* END    DEFINE_FUNCTION  Send_GET_method  			        *)
(************************************************************************)
(************************************************************************)
(* START  DEFINE_FUNCTION  Send_PUT_method  			        *)
(************************************************************************)
DEFINE_FUNCTION Send_PUT_method(CHAR cURI[], CHAR cJson_Parameter_String[200])
  {
  STACK_VAR CHAR cPUT_Cmd[500]
  
  cPUT_Cmd = "'PUT ', cURI,' HTTP/1.1', CrLf,
              'Content-type: application/json', CrLf,
	      'Connection: keep-alive', CrLf,
	      'Authorization: ',sAuthorization.cType,' ',sAuthorization.cCredentials, CrLf,
	      'Content-Length: ',ITOA(LENGTH_ARRAY(cJson_Parameter_String)),CrLf,
	      'Host: ',cIpAddress,':',ITOA(nSPKR_IP_PORT),CrLf,
	       CrLf,    
	       cJson_Parameter_String"
	       
  WriteToTxBuffer(cPUT_Cmd)
  
  // STRING_TO_DIAGNOSTICS("'SPKR_DEBUG: Send_PUT_method: ',cURI,', JSON=',cJson_Parameter_String", 12, __LINE__)
  }
(************************************************************************)
(* END    DEFINE_FUNCTION  Send_PUT_method  			        *)
(************************************************************************)
(************************************************************************)
(* START  DEFINE_FUNCTION  WriteToTxBuffer  			        *)
(************************************************************************)
DEFINE_FUNCTION WriteToTxBuffer(CHAR cCmd[])
  {
  // DEBUG("'Write to TX Buffer: ',cCmd", __LINE__)
  cSPKR_TX_BUFFER = "cSPKR_TX_BUFFER, cCmd, $ff"
  }
(************************************************************************)
(* END    DEFINE_FUNCTION  WriteToTxBuffer  			        *)
(************************************************************************)
(************************************************************************)
(* START  DEFINE_FUNCTION  SendFromTxBufferToDevice      	        *)
(************************************************************************)
DEFINE_FUNCTION SendFromTxBufferToDevice()
  {
  STACK_VAR CHAR cCmdToSend[500]
   
  SELECT
    {
    ACTIVE(sSpkrStatus.nIsOnline = 0):
      {
      WAIT 100 'try_to_connect'
	{
	IP_CLIENT_OPEN(dvSpkr___1.port, cIpAddress, nSPKR_IP_PORT, IP_TCP) 
	DEBUG("'IP_CLIENT_OPEN in SendFromTxBufferToDevice()'", __LINE__)
	}
      }
    ACTIVE(FIND_STRING(cSPKR_TX_BUFFER, "$ff", 1)):
      {
      cCmdToSend = REMOVE_STRING(cSPKR_TX_BUFFER, "$ff", 1)
      cCmdToSend = LEFT_STRING(cCmdToSend, LENGTH_ARRAY(cCmdToSend) - 1)

      SEND_STRING dvSpkr___1, "cCmdToSend"  
      
      CANCEL_WAIT 'send_ready'
      nSPKR_send_ready = 0
      WAIT 50 'send_ready'
	{
	nSPKR_send_ready = 1
	DEBUG("'-------- TX: send_ready set to 1 by timeout 5 sec'", __LINE__)
	}
      
      IF(nPollingPause < 2)
	{
	nPollingPause = 2       
	}
	
//    nTxCounter++
//    cLast_TX = "'Last_TX #',ITOA(nTxCounter),' ->' ,cCmdToSend"

      DEBUG("'<<<<<<<< TX: ',cCmdToSend", __LINE__)
      }
    }
    
  }// end function
(************************************************************************)
(* END   DEFINE_FUNCTION  SendFromTxBufferToDevice		        *)
(************************************************************************)
(************************************************************************)
(* START  DEFINE_FUNCTION  ReadRxBuffer  			        *)
(************************************************************************)
DEFINE_FUNCTION ReadRxBuffer()
  {
  STACK_VAR INTEGER i

  FOR(i = 1; i <= 100; i++)
    {
    IF(FIND_STRING(cSPKR_RX_BUFFER, "$0A", 1))
       {
       ProcessOneLine(REMOVE_STRING(cSPKR_RX_BUFFER, "$0A", 1))
       }   
    ELSE
       {
       BREAK
       }
    }
  }   
(************************************************************************)
(* END    DEFINE_FUNCTION  ReadRxBuffer  			        *)
(************************************************************************)
(************************************************************************)
(* START  DEFINE_FUNCTION  ProcessOneLine			        *)
(************************************************************************)
//
//  Example for received line:
//  HTTP/1.1 200 OK$0D$0AContent-Type: application/json$0D$0AContent-Length: 26$0D$0A$0D$0A{"level":-30,"mute":false}$0D$0A
//
//  Request to Speaker:               Response from Speaker:
//  GET /public/v1/device/pwr    -->  {"state":"ACTIVE","poeAllocatedPwr":25.5,"poePd15W":false}$0D$0A 
//  GET /public/v1/audio/volume  -->  {"level":-30,"mute":false}$0D$0A
//  GET /public/v1/audio/inputs  -->  {"input":["AoIP01","AoIP02"]}$0D$0A 
//  GET /public/v1/events        -->  {"uptime":"8h 40m 40s","bsLevel":-120.3,"twCoilT":46,"twLevel":-108.7,"inLevel":-102.6,"nwInKbps":0,"cpuLoad":69,"cpuT":46.1}$0D$0A 
//  GET /public/v1/network/zone  -->  {"zone":0,"name":""}$0D$0A
//  GET /public/v1/profile/list  -->  {"selected":1,"startup":1,"list":[{"id":1,"name":"Profile 1"},{"id":2,"name":"Profile 2"}]}$0D$0A 
//  GET /public/v1/device/id     -->  {"mac":"AC:47:23:20:03:AA","barcode":"4420AWMa1100078","hwId":"","model":"4420","modId":"4420","calibr":"20200615-1722"}$0D$0A
//  GET /public/v1/device/info   -->  {"fwId":"44x0-1.2.10-202201180940","build":"b37370","baseId":"1.0.0","hwId":"","model":"4420","category":"SAM_2WAY","technology":"SAM_IP","upgradeId":10,"apiVer":"v1"}$0D$0A
//
//
//  single parameter view:
//
//  GET /public/v1/device/pwr    -->  {"state":"ACTIVE"} 
//  GET /public/v1/device/pwr    -->  {"poeAllocatedPwr":25.5} 
//  GET /public/v1/device/pwr    -->  {"poePd15W":false} 

//  GET /public/v1/audio/volume  -->  {"level":-30,"mute":false} 
//  GET /public/v1/audio/volume  -->  {"mute":false} 

//  GET /public/v1/audio/inputs  -->  {"input":["AoIP01","AoIP02"]}  

//  GET /public/v1/events        -->  {"uptime":"8h 40m 40s"} 
//  GET /public/v1/events        -->  {"bsLevel":-120.3} 
//  GET /public/v1/events        -->  {"twCoilT":46} 
//  GET /public/v1/events        -->  {"twLevel":-108.7} 
//  GET /public/v1/events        -->  {"inLevel":-102.6} 
//  GET /public/v1/events        -->  {"nwInKbps":0} 
//  GET /public/v1/events        -->  {"cpuLoad":69} 
//  GET /public/v1/events        -->  {"cpuT":46.1} 

//  GET /public/v1/network/zone  -->  {"zone":0}
//  GET /public/v1/network/zone  -->  {"name":""}

//  GET /public/v1/profile/list  -->  {"selected":1} 
//  GET /public/v1/profile/list  -->  {"startup":1} 
//  GET /public/v1/profile/list  -->  {"list":[{"id":1,"name":"Profile 1"},{"id":2,"name":"Profile 2"}]} 

//  GET /public/v1/device/id     -->  {"mac":"AC:47:23:20:03:AA"}
//  GET /public/v1/device/id     -->  {"barcode":"4420AWMa1100078"}
//  GET /public/v1/device/id     -->  {"hwId":""}
//  GET /public/v1/device/id     -->  {"model":"4420"}
//  GET /public/v1/device/id     -->  {"modId":"4420"}
//  GET /public/v1/device/id     -->  {"calibr":"20200615-1722"}

//  GET /public/v1/device/info   -->  {"fwId":"44x0-1.2.10-202201180940"}
//  GET /public/v1/device/info   -->  {"build":"b37370"}
//  GET /public/v1/device/info   -->  {"baseId":"1.0.0"}
//  GET /public/v1/device/info   -->  {"hwId":""}
//  GET /public/v1/device/info   -->  {"model":"4420"}
//  GET /public/v1/device/info   -->  {"category":"SAM_2WAY"}
//  GET /public/v1/device/info   -->  {"technology":"SAM_IP"}
//  GET /public/v1/device/info   -->  {"upgradeId":10}
//  GET /public/v1/device/info   -->  {"apiVer":"v1"}


DEFINE_FUNCTION ProcessOneLine(CHAR cOneLine[])
  {
  STACK_VAR CHAR cTemp[100]
  LOCAL_VAR INTEGER nHTTP_OK_counter           // must be LOCAL_VAR
  
  
  
  DEBUG("'>>>>>>>> RX: ',cOneLine", __LINE__)
  
  IF(FIND_STRING(cOneLine, '{', 1) = 1)
    {
    // STRING_TO_DIAGNOSTICS("'SPKR_DEBUG: ProcessOneLine: ',cOneLine", 12, __LINE__)
    }
  
  SELECT
    {
    ACTIVE(FIND_STRING(cOneLine, "'HTTP/1.1 200 OK',$0D,$0A", 1) = 1):         // wenn folgender String vorhanden ist:   HTTP/1.1 200 OK$0D$0A
      {
      CANCEL_WAIT 'send_ready'
      nSPKR_send_ready = 1
      nHTTP_OK_counter++
      }
    ACTIVE((FIND_STRING(cOneLine, '{', 1) = 0)  OR   
           (FIND_STRING(cOneLine, '}', 1) = 0)   ): 
      {
      // not nothing, not a JSON expression
      }
      
    // ------------------- device/pwr -----------------------------------------
    ACTIVE(FIND_STRING(cOneLine, '"state":', 1) > 0):                          // {"state":"ACTIVE","poeAllocatedPwr":25.5,"poePd15W":false}
      {
      LOCAL_VAR INTEGER nGetPwrActive_Ctrl                                     // to avoid an endless loop of get device/pwr 
      LOCAL_VAR INTEGER nGetPwrStandby_Ctrl                                    // to avoid an endless loop of get device/pwr 
      
      // extract power state:
      cTemp = GET_KEY_VAL_FROM_JSON(cOneLine, 'state', 1, 1)
      sSpkrStatus.cDevice_pwr_State = cTemp
      SELECT
	{
	ACTIVE(cTemp = 'ACTIVE'):
	  {
	  sSpkrStatus.nPOWER_FB = 1
	  nGetPwrActive_Ctrl = 0
	  nGetPwrStandby_Ctrl = 0
	  }
	ACTIVE(cTemp = 'STANDBY'):
	  {
	  sSpkrStatus.nPOWER_FB = 0
	  nGetPwrActive_Ctrl = 0
	  nGetPwrStandby_Ctrl = 0
	  }
	ACTIVE(cTemp = 'WAKEUP'):
	  {
	  nGetPwrStandby_Ctrl = 0
	  IF(nGetPwrActive_Ctrl < 4)                                           // repeat 'get device/pwr' max. 4 times at state 'WAKEUP'
	    {
	    WAIT 10
	      {
	      Send_GET_method('/public/v1/device/pwr')
	      nGetPwrActive_Ctrl++
	      }
	    }
	  }
	ACTIVE(cTemp = 'WAIT_STANDBY'):
	  {
	  nGetPwrActive_Ctrl = 0
	  IF(nGetPwrStandby_Ctrl < 4)                                          // repeat 'get device/pwr' max. 4 times at state 'WAIT_STANDBY'
	    {
	    WAIT 10
	      {
	      Send_GET_method('/public/v1/device/pwr')
	      nGetPwrStandby_Ctrl++
	      }
	    }
	  }
	}
	
      // extract poeAllocatedPwr state:
      cTemp = GET_KEY_VAL_FROM_JSON(cOneLine, 'poeAllocatedPwr', 1, 1)
      sSpkrStatus.fDevice_pwr_PoeAllocatedPwr = ATOF(cTemp)
      
      // extract poePd15W state:
      cTemp = GET_KEY_VAL_FROM_JSON(cOneLine, 'poePd15W', 1, 1)
      SELECT
	{
	ACTIVE(cTemp = 'true'):
	  {
	  sSpkrStatus.nDevice_pwr_PoePd15W = 1
	  }
	ACTIVE(cTemp = 'false'):
	  {
	  sSpkrStatus.nDevice_pwr_PoePd15W = 0
	  }
	}
      }
      
    // ------------------- audio/volume -----------------------------------------
    ACTIVE(FIND_STRING(cOneLine, '"level":', 1) > 0):                          // {"level":-31,"mute":true}
      {
      STACK_VAR FLOAT fTempVol
      
      // extract level:
      cTemp = GET_KEY_VAL_FROM_JSON(cOneLine, 'level', 1, 1)
      fTempVol = ATOF(cTemp)
      SELECT
	{
	ACTIVE((fTempVol >= -200) AND (fTempVol <= 0)):
	  {
	  STACK_VAR SINTEGER snTempLevel
	  sSpkrStatus.fVolume = fTempVol
	  snTempLevel = TYPE_CAST(sSpkrStatus.fVolume*10)
	  SEND_LEVEL vdvSpkr__1, VOL_LVL, snTempLevel
	  }
	ACTIVE(1):
	  {
	  // error
	  }
	}
	
      // extract mute state:
      cTemp = GET_KEY_VAL_FROM_JSON(cOneLine, 'mute', 1, 1)
      SELECT
	{
	ACTIVE(cTemp ='true'):
	  {
	  sSpkrStatus.cMute = 'true'
	  }
	ACTIVE(cTemp ='false'):
	  {
	  sSpkrStatus.cMute = 'false'
	  }
	}
      }
      
    // ------------------- audio/inputs -----------------------------------------
    ACTIVE(FIND_STRING(cOneLine, '"input":', 1) > 0):                          //  {"input":["AoIP01","AoIP02"]}    or    ["A","AoIP01","AoIP02"]
      {
      // extract input:
      cTemp = GET_KEY_VAL_FROM_JSON(cOneLine, 'input', 1, 1)
      IF(FIND_STRING(cTemp, '"A"', 1) > 1)       { sSpkrStatus.nInput_Analog = 1 }
      ELSE                                       { sSpkrStatus.nInput_Analog = 0 }
      IF(FIND_STRING(cTemp, '"AoIP01"', 1) > 1)  { sSpkrStatus.nInput_AoIP01 = 1 }
      ELSE                                       { sSpkrStatus.nInput_AoIP01 = 0 }
      IF(FIND_STRING(cTemp, '"AoIP02"', 1) > 1)  { sSpkrStatus.nInput_AoIP02 = 1 }
      ELSE                                       { sSpkrStatus.nInput_AoIP02 = 0 }
      }
      
    // ------------------- events -----------------------------------------
    ACTIVE(FIND_STRING(cOneLine, '"uptime":', 1) > 0):                          //  {"uptime":"8h 40m 40s","bsLevel":-120.3,"twCoilT":46,"twLevel":-108.7,"inLevel":-102.6,"nwInKbps":0,"cpuLoad":69,"cpuT":46.1}
      {
      cTemp = GET_KEY_VAL_FROM_JSON(cOneLine, 'uptime', 1, 1)
      sSpkrStatus.cEvents_uptime = cTemp
      cTemp = GET_KEY_VAL_FROM_JSON(cOneLine, 'bsLevel', 1, 1)
      sSpkrStatus.fEvents_bsLevel = ATOF(cTemp)
      cTemp = GET_KEY_VAL_FROM_JSON(cOneLine, 'twCoilT', 1, 1)
      sSpkrStatus.fEvents_twCoilT = ATOF(cTemp)
      cTemp = GET_KEY_VAL_FROM_JSON(cOneLine, 'twLevel', 1, 1)
      sSpkrStatus.fEvents_twLevel = ATOF(cTemp)
      cTemp = GET_KEY_VAL_FROM_JSON(cOneLine, 'inLevel', 1, 1)
      sSpkrStatus.fEvents_inLevel = ATOF(cTemp)
      cTemp = GET_KEY_VAL_FROM_JSON(cOneLine, 'nwInKbps', 1, 1)
      sSpkrStatus.fEvents_nwInKbps = ATOF(cTemp)
      cTemp = GET_KEY_VAL_FROM_JSON(cOneLine, 'cpuLoad', 1, 1)
      sSpkrStatus.fEvents_cpuLoad = ATOF(cTemp)
      cTemp = GET_KEY_VAL_FROM_JSON(cOneLine, 'cpuT', 1, 1)
      sSpkrStatus.fEvents_cpuT = ATOF(cTemp)
      }
      
    // ------------------- network/zone -----------------------------------------
    ACTIVE(FIND_STRING(cOneLine, '"zone":', 1) > 0):                           //  {"zone":0,"name":""}
      {
      cTemp = GET_KEY_VAL_FROM_JSON(cOneLine, 'zone', 1, 1)
      sSpkrStatus.nNetwork_zone_zone = ATOI(cTemp)
      cTemp = GET_KEY_VAL_FROM_JSON(cOneLine, 'name', 1, 1)
      sSpkrStatus.cNetwork_zone_name = cTemp
      }
      
    // ------------------- profile/list -----------------------------------------
    ACTIVE(FIND_STRING(cOneLine, '"selected":', 1) > 0):                       //  {"selected":1,"startup":1,"list":[{"id":1,"name":"Profile 1"},{"id":2,"name":"Profile 2"}]}
      {
      STACK_VAR CHAR cList_temp[450]
      STACK_VAR INTEGER nListIndex                 // 1..6
      cTemp = GET_KEY_VAL_FROM_JSON(cOneLine, 'selected', 1, 1)
      sSpkrStatus.nProfile_list_selected = ATOI(cTemp)
      cTemp = GET_KEY_VAL_FROM_JSON(cOneLine, 'startup', 1, 1)
      sSpkrStatus.nProfile_list_startup = ATOI(cTemp)
      cList_temp = GET_KEY_VAL_FROM_JSON(cOneLine, 'list', 1, 1)               // [{"id":1,"name":"Profile 1"},{"id":2,"name":"Profile 2"}]
      
      sSpkrStatus.cProfile_list_name[1] = ''
      sSpkrStatus.cProfile_list_name[2] = ''
      sSpkrStatus.cProfile_list_name[3] = ''
      sSpkrStatus.cProfile_list_name[4] = ''
      sSpkrStatus.cProfile_list_name[5] = ''
      sSpkrStatus.cProfile_list_name[6] = ''
      
      nListIndex = ATOI(GET_KEY_VAL_FROM_JSON(cList_temp, 'id', 1, 1))
      IF( (nListIndex >= 1) AND (nListIndex <= 6) )                            // if there is a key/val 'id' found
	{
	sSpkrStatus.cProfile_list_name[nListIndex] = GET_KEY_VAL_FROM_JSON(cList_temp, 'name', 1, 1)
	}
	
      nListIndex = ATOI(GET_KEY_VAL_FROM_JSON(cList_temp, 'id', 2, 1))
      IF( (nListIndex >= 1) AND (nListIndex <= 6) )                            // if there is a 2. key/val 'id' found
	{
	sSpkrStatus.cProfile_list_name[nListIndex] = GET_KEY_VAL_FROM_JSON(cList_temp, 'name', 2, 1)
	}
	
      nListIndex = ATOI(GET_KEY_VAL_FROM_JSON(cList_temp, 'id', 3, 1))
      IF( (nListIndex >= 1) AND (nListIndex <= 6) )                            // if there is a 3. key/val 'id' found
	{
	sSpkrStatus.cProfile_list_name[nListIndex] = GET_KEY_VAL_FROM_JSON(cList_temp, 'name', 3, 1)
	}
	
      nListIndex = ATOI(GET_KEY_VAL_FROM_JSON(cList_temp, 'id', 4, 1))
      IF( (nListIndex >= 1) AND (nListIndex <= 6) )                            // if there is a 4. key/val 'id' found
	{
	sSpkrStatus.cProfile_list_name[nListIndex] = GET_KEY_VAL_FROM_JSON(cList_temp, 'name', 4, 1)
	}
	
      nListIndex = ATOI(GET_KEY_VAL_FROM_JSON(cList_temp, 'id', 5, 1))
      IF( (nListIndex >= 1) AND (nListIndex <= 6) )                            // if there is a 5. key/val 'id' found
	{
	sSpkrStatus.cProfile_list_name[nListIndex] = GET_KEY_VAL_FROM_JSON(cList_temp, 'name', 5, 1)
	}
	
      nListIndex = ATOI(GET_KEY_VAL_FROM_JSON(cList_temp, 'id', 6, 1))
      IF( (nListIndex >= 1) AND (nListIndex <= 6) )                            // if there is a 6. key/val 'id' found
	{
	sSpkrStatus.cProfile_list_name[nListIndex] = GET_KEY_VAL_FROM_JSON(cList_temp, 'name', 6, 1)
	}
	
      SEND_STRING vdvSpkr__1,"'DEVICE_INFO-/profile/list,selected,',ITOA(sSpkrStatus.nProfile_list_selected)"     // ensure UI knows the selected profile
      }
      
    // ------------------- device/id  -----------------------------------------
    ACTIVE(FIND_STRING(cOneLine, '"mac":', 1) > 0):                            //  {"mac":"AC:47:23:20:03:AA","barcode":"4420AWMa1100078","hwId":"","model":"4420","modId":"4420","calibr":"20200615-1722"}
      {
      cTemp = GET_KEY_VAL_FROM_JSON(cOneLine, 'mac', 1, 1)
      sSpkrStatus.cDevice_id_mac = cTemp
      cTemp = GET_KEY_VAL_FROM_JSON(cOneLine, 'barcode', 1, 1)
      sSpkrStatus.cDevice_id_barcode = cTemp
      cTemp = GET_KEY_VAL_FROM_JSON(cOneLine, 'hwId', 1, 1)
      sSpkrStatus.cDevice_id_hwId = cTemp
      cTemp = GET_KEY_VAL_FROM_JSON(cOneLine, 'model', 1, 1)
      sSpkrStatus.cDevice_id_model = cTemp
      cTemp = GET_KEY_VAL_FROM_JSON(cOneLine, 'modId', 1, 1)
      sSpkrStatus.cDevice_id_modId = cTemp
      cTemp = GET_KEY_VAL_FROM_JSON(cOneLine, 'calibr', 1, 1)
      sSpkrStatus.cDevice_id_calibr = cTemp
      }
      
      
    // ------------------- device/info  -----------------------------------------
    ACTIVE(FIND_STRING(cOneLine, '"fwId":', 1) > 0):                           //  {"fwId":"44x0-1.2.10-202201180940","build":"b37370","baseId":"1.0.0","hwId":"","model":"4420","category":"SAM_2WAY","technology":"SAM_IP","upgradeId":10,"apiVer":"v1"}
      {
      cTemp = GET_KEY_VAL_FROM_JSON(cOneLine, 'fwId', 1, 1)
      sSpkrStatus.cDevice_info_fwId = cTemp
      cTemp = GET_KEY_VAL_FROM_JSON(cOneLine, 'build', 1, 1)
      sSpkrStatus.cDevice_info_build = cTemp
      cTemp = GET_KEY_VAL_FROM_JSON(cOneLine, 'baseId', 1, 1)
      sSpkrStatus.cDevice_info_baseId = cTemp
      cTemp = GET_KEY_VAL_FROM_JSON(cOneLine, 'hwId', 1, 1)
      sSpkrStatus.cDevice_info_hwId = cTemp
      cTemp = GET_KEY_VAL_FROM_JSON(cOneLine, 'model', 1, 1)
      sSpkrStatus.cDevice_info_model = cTemp
      cTemp = GET_KEY_VAL_FROM_JSON(cOneLine, 'category', 1, 1)
      sSpkrStatus.cDevice_info_category = cTemp
      cTemp = GET_KEY_VAL_FROM_JSON(cOneLine, 'technology', 1, 1)
      sSpkrStatus.cDevice_info_technology = cTemp
      cTemp = GET_KEY_VAL_FROM_JSON(cOneLine, 'upgradeId', 1, 1)
      sSpkrStatus.cDevice_info_upgradeId = cTemp
      cTemp = GET_KEY_VAL_FROM_JSON(cOneLine, 'apiVer', 1, 1)
      sSpkrStatus.cDevice_info_apiVer = cTemp
      cTemp = GET_KEY_VAL_FROM_JSON(cOneLine, 'confirmFwUpdate', 1, 1)
      sSpkrStatus.nDevice_info_confirmFwUpdate = ATOI(cTemp)
      }
    }

  }
(************************************************************************)
(* END    DEFINE_FUNCTION  ProcessOneLine			        *)
(************************************************************************)
(************************************************************************)
(* START  DEFINE_FUNCTION  GET_KEY_VAL_FROM_JSON                        *)
(************************************************************************)
// 24.09.2020/CL
// 
// Examples based on following JSON expression:
// cJsonString = "'{'                                      ,CrLf,
//		  '  "firstName": "John" ,'                ,CrLf,
//		  '  "lastName":  "Smith"  ,'              ,CrLf,
//		  '  "isAlive": true,'                     ,CrLf,
//		  '  "age": 27,'                           ,CrLf,
//		  '  "address": {'                         ,CrLf,
//		  '    "streetAddress": "21 2nd Street",'  ,CrLf,
//		  '    "city": "New York",'                ,CrLf,
//		  '    "state": "NY",'                     ,CrLf,
//		  '    "postalCode": "10021-3100"'         ,CrLf,
//		  '  },'                                   ,CrLf,
//		  '  "phoneNumbers": ['                    ,CrLf,
//		  '    {'                                  ,CrLf,
//		  '      "type": "home",'                  ,CrLf,
//		  '      "number": "212 555-1234"'         ,CrLf,
//		  '    },'                                 ,CrLf,
//		  '    {'                                  ,CrLf,
//		  '      "type": "office",'                ,CrLf,
//		  '      "number": "646 555-4567"'         ,CrLf,
//		  '    }'                                  ,CrLf,
//		  '  ],'                                   ,CrLf,
//		  '  "children": true,'                    ,CrLf,
//		  '  "spouse": null'                       ,CrLf,
//		  '}'"
//
// GET_KEY_VAL_FROM_JSON(cJsonString, 'lastName', 1, 1)
// Returnvalue  -> Smith 
//
// GET_KEY_VAL_FROM_JSON(cJsonString, 'lastName', 1, 0)
// Returnvalue  -> "Smith"                             
//
// GET_KEY_VAL_FROM_JSON(cJsonString, 'children', 1, 1)
// Returnvalue  -> true
//
// GET_KEY_VAL_FROM_JSON(cJsonString, 'spouse', 1, 1)
// Returnvalue  -> null
//
// GET_KEY_VAL_FROM_JSON(cJsonString, 'pet', 1, 1) 
// Returnvalue  -> NOT_FOUND
//
// GET_KEY_VAL_FROM_JSON(cJsonString, 'address', 1, 1)
// Returnvalue  -> {      "streetAddress": "21 2nd Street",      "city": "New York",      "state": "NY",      "postalCode": "10021-3100"    }
//
// GET_KEY_VAL_FROM_JSON('{      "streetAddress": "21 2nd Street",      "city": "New York",      "state": "NY",      "postalCode": "10021-3100"    }', 'city', 1, 1)
// Returnvalue  -> New York
//
// GET_KEY_VAL_FROM_JSON(cJsonString, 'city', 1, 1)
// Returnvalue  -> New York
//
// GET_KEY_VAL_FROM_JSON(cJsonString, 'phoneNumbers', 1, 1)
// Returnvalue  -> [      {        "type": "home",        "number": "212 555-1234"      },      {        "type": "office",        "number": "646 555-4567"      }    ]
//
// GET_KEY_VAL_FROM_JSON([      {        "type": "home",        "number": "212 555-1234"      },      {        "type": "office",        "number": "646 555-4567"      }    ], 'number', 1, 1)
// Returnvalue  -> 212 555-1234
//
// GET_KEY_VAL_FROM_JSON([      {        "type": "home",        "number": "212 555-1234"      },      {        "type": "office",        "number": "646 555-4567"      }    ], 'number', 2, 1)
// Returnvalue  -> 646 555-4567
//
// GET_KEY_VAL_FROM_JSON(cJsonString, 'number', 1, 1)
// Returnvalue  -> 212 555-1234
//
// GET_KEY_VAL_FROM_JSON(cJsonString, 'number', 2, 1) 
// Returnvalue  -> 646 555-4567
//
//
DEFINE_FUNCTION CHAR[200] GET_KEY_VAL_FROM_JSON(CHAR cJsonString[], CHAR cKey[50], INTEGER nKeyNumber, INTEGER nRemoveDoubleQuotes)
  {
  STACK_VAR CHAR    cJsonString_local[1000]
  STACK_VAR CHAR    cKey_local[53]
  STACK_VAR INTEGER nKeyNumberLoop
  STACK_VAR INTEGER nKeyColonPos               
  STACK_VAR INTEGER nStringPos			
  STACK_VAR INTEGER nSquaredBracketCounter     
  STACK_VAR INTEGER nCurlyBracketCounter       
  STACK_VAR CHAR    cReturnValue[250]
  STACK_VAR INTEGER cReturnStartPos_WO_space   
  STACK_VAR INTEGER cReturnEndPos_WO_space     

  
  nSquaredBracketCounter = 0
  nCurlyBracketCounter = 0
  
  cJsonString_local = cJsonString
  cReturnValue = 'NOT_FOUND'
  
  
  
  /////////////////////////////////////////////////////////////////////////////
  cKey_local = "'"',cKey,'":'"
  nKeyColonPos = 0
  FOR(nKeyNumberLoop=1; nKeyNumberLoop<=nKeyNumber; nKeyNumberLoop++)
    {
    nKeyColonPos++
    nKeyColonPos = FIND_STRING(cJsonString_local, cKey_local, nKeyColonPos)
    }
  
  IF(nKeyColonPos = 0)
    {
    RETURN cReturnValue
    }
  
  nKeyColonPos = nKeyColonPos+LENGTH_ARRAY(cKey)+2
  
  
  
  /////////////////////////////////////////////////////////////////////////////
  FOR(nStringPos = nKeyColonPos+1; nStringPos<=LENGTH_ARRAY(cJsonString_local); nStringPos++)            
    {
    //SEND_STRING 0, "'---> nStringPos = ',ITOA(nStringPos), ' ---> cJsonString_local[nStringPos] = ',cJsonString_local[nStringPos]"
    SELECT
      {
      ACTIVE(((cJsonString_local[nStringPos] = ',') OR (cJsonString_local[nStringPos] = '}')) AND
             (nSquaredBracketCounter = 0)                                                     AND
	     (nCurlyBracketCounter = 0)                                                         ):
	{
	cReturnValue = MID_STRING(cJsonString_local, nKeyColonPos+1, nStringPos-nKeyColonPos-1)
	BREAK
	}
      ACTIVE(cJsonString_local[nStringPos] = '['):
	{
	nSquaredBracketCounter++
	}
      ACTIVE(cJsonString_local[nStringPos] = ']'):
	{
	nSquaredBracketCounter--
	}
      ACTIVE(cJsonString_local[nStringPos] = '{'):
	{
	nCurlyBracketCounter++
	}
      ACTIVE(cJsonString_local[nStringPos] = '}'):
	{
	nCurlyBracketCounter--
	}
      }
    }
  
  
  
  /////////////////////////////////////////////////////////////////////////////
  REPLACE_IN_STRING(cReturnValue, $0D, ' ')  
  REPLACE_IN_STRING(cReturnValue, $0A, ' ')  
  
  
  
  /////////////////////////////////////////////////////////////////////////////
  cReturnStartPos_WO_space = 0
  cReturnEndPos_WO_space = 0
  FOR(nStringPos=1; nStringPos<=LENGTH_ARRAY(cReturnValue); nStringPos++)
    {
    IF( NOT(cReturnValue[nStringPos] = ' '))
      {
      cReturnStartPos_WO_space = nStringPos
      BREAK
      }
    }
  FOR(nStringPos=LENGTH_ARRAY(cReturnValue); nStringPos>=1; nStringPos--)
    {
    IF( NOT(cReturnValue[nStringPos] = ' '))
      {
      cReturnEndPos_WO_space = nStringPos
      BREAK
      }
    }
  IF((cReturnStartPos_WO_space > 0)  AND
     (cReturnEndPos_WO_space > 0)      )
    {
    cReturnValue = MID_STRING(cReturnValue, cReturnStartPos_WO_space, cReturnEndPos_WO_space-cReturnStartPos_WO_space+1)
    }
  
  
  
  /////////////////////////////////////////////////////////////////////////////
  IF(nRemoveDoubleQuotes = 1)
    {
    IF(cReturnValue[1] = '"')
      {
      GET_BUFFER_STRING(cReturnValue, 1)
      }
    IF(cReturnValue[LENGTH_ARRAY(cReturnValue)] = '"')
      {
      SET_LENGTH_ARRAY(cReturnValue, LENGTH_ARRAY(cReturnValue)-1 )
      }
    }

  RETURN cReturnValue
  }
(************************************************************************)
(* END    DEFINE_FUNCTION  GET_KEY_VAL_FROM_JSON                        *)
(************************************************************************)
(************************************************************************)
(* START  DEFINE_FUNCTION  DEBUG			                *)
(************************************************************************)
//
// cMessage:  Text to write in Diagnostics
// nCFLN:     Call From Line
//
DEFINE_FUNCTION DEBUG(CHAR cMessage[], INTEGER nCFLN)
  {
  IF(nSPKR_DEBUG)
    {
    STRING_TO_DIAGNOSTICS("'SPKR_DEBUG: ',cMessage", 12, nCFLN)
    }
  }
(************************************************************************)
(* END    DEFINE_FUNCTION  DEBUG			                *)
(************************************************************************)
(*****************************************************************************)
(* START  DEFINE_FUNCTION  GET_FROM_DATATEXT                                 *)
(*****************************************************************************)
// REV10 (version for 4 parameters)
//
DEFINE_FUNCTION CHAR[100] GET_FROM_DATATEXT(CHAR cDATATEXT[], INTEGER nRETURN_WHAT, CHAR cCMD_SEPARATOR[], CHAR cPARAM_SEPARATOR[])
  {
  STACK_VAR CHAR cCMD[30]    
  STACK_VAR CHAR cPARAM[100] 
  STACK_VAR CHAR cRemString[100]
  STACK_VAR INTEGER nEscPos                        
  STACK_VAR INTEGER nEscFound
  
  cCMD   = ''
  cPARAM = ''
  cRemString = ''
  nEscFound = 0
  
  // 
  nEscPos = FIND_STRING(cDATATEXT, "$1B,cPARAM_SEPARATOR", 1) 
  WHILE(nEscPos > 0)                                                           // nEscPos = 16
    {
    nEscFound = 1
    cDATATEXT = "LEFT_STRING(cDATATEXT,nEscPos),RIGHT_STRING(cDATATEXT, LENGTH_ARRAY(cDATATEXT)-nEscPos-1)" 
    nEscPos = FIND_STRING(cDATATEXT, "$1B,cPARAM_SEPARATOR", 1)
    }
  
  
  SWITCH(nRETURN_WHAT)
    {
    ///////////////////////////////////////////////////////////////////////////
    // nRETURN_WHAT: Command Keyword:
    CASE 0:
      {
      // extract Command 
      IF(FIND_STRING(cDATATEXT, cCMD_SEPARATOR, 1) = 0) 
	{
	RETURN cDATATEXT
	}
      
      cCMD = REMOVE_STRING(cDATATEXT,cCMD_SEPARATOR,1)                         // INPUT-45,0,50    --> cCMD = INPUT-     --> cDATATEXT = 45,0,50
      cCMD = LEFT_STRING(cCMD, LENGTH_STRING(cCMD)-1)
      RETURN cCMD
      }
    ///////////////////////////////////////////////////////////////////////////
    // nRETURN_WHAT: Parameter 1:
    CASE 1:
      {
      // remove Command
      cRemString = REMOVE_STRING(cDATATEXT,cCMD_SEPARATOR,1)                   // INPUT-45,0,50    --> cCMD = INPUT-     --> cDATATEXT = 45,0,50
     
      // 
      IF(cRemString = '')
	{
	RETURN cPARAM
	}
     
      // extract Parameter 1
      IF(FIND_STRING(cDATATEXT, cPARAM_SEPARATOR, 1) > 0)
	{
	cPARAM = REMOVE_STRING(cDATATEXT,cPARAM_SEPARATOR,1)                   // cDATATEXT = 45,0,50   --> cPARAM1 = 45,    --> cDATATEXT = 0,50    
	cPARAM = LEFT_STRING(cPARAM, LENGTH_STRING(cPARAM)-1)
	IF(nEscFound == 1)  { REPLACE_IN_STRING(cPARAM, $1B, cPARAM_SEPARATOR[1]) }
	RETURN cPARAM
	}
      ELSE
	{
	cPARAM = cDATATEXT  
	IF(nEscFound == 1)  { REPLACE_IN_STRING(cPARAM, $1B, cPARAM_SEPARATOR[1]) }
	RETURN cPARAM
	}
      }
    ///////////////////////////////////////////////////////////////////////////
    // nRETURN_WHAT: Parameter 2:
    CASE 2:
      {
      // remove Command
      REMOVE_STRING(cDATATEXT,cCMD_SEPARATOR,1)                                // INPUT-45,0,50    --> cCMD = INPUT-     --> cDATATEXT = 45,0,50                     

      // remove Parameter 1
      cRemString = REMOVE_STRING(cDATATEXT,cPARAM_SEPARATOR,1)                 // --> cDATATEXT = 0,50                             

      // 
      IF(cRemString = '')
	{
	RETURN cPARAM
	}
     
      // extract Parameter 2 
      IF(FIND_STRING(cDATATEXT, cPARAM_SEPARATOR, 1) > 0)
	{
	cPARAM = REMOVE_STRING(cDATATEXT,cPARAM_SEPARATOR,1)                   // cDATATEXT = 45,0,50   --> cPARAM1 = 45,    --> cDATATEXT = 0,50    
	cPARAM = LEFT_STRING(cPARAM, LENGTH_STRING(cPARAM)-1)
	IF(nEscFound == 1)  { REPLACE_IN_STRING(cPARAM, $1B, cPARAM_SEPARATOR[1]) }
	RETURN cPARAM
	}
      ELSE
	{
	cPARAM = cDATATEXT 
	IF(nEscFound == 1)  { REPLACE_IN_STRING(cPARAM, $1B, cPARAM_SEPARATOR[1]) }
	RETURN cPARAM
	}
      }
    ///////////////////////////////////////////////////////////////////////////
    // nRETURN_WHAT: Parameter 3:
    CASE 3:
      {
      // remove Command
      REMOVE_STRING(cDATATEXT,cCMD_SEPARATOR,1)                                // INPUT-45,0,50    --> cCMD = INPUT-     --> cDATATEXT = 45,0,50 

      // remove Parameter 1
      REMOVE_STRING(cDATATEXT,cPARAM_SEPARATOR,1)                              // --> cDATATEXT = 0,50            

      // remove Parameter 2
      cRemString = REMOVE_STRING(cDATATEXT,cPARAM_SEPARATOR,1)                 // --> cDATATEXT = 50     
     
      //
      IF(cRemString = '')
	{
	RETURN cPARAM
	}
     
      // extract Parameter 3 
      IF(FIND_STRING(cDATATEXT, cPARAM_SEPARATOR, 1) > 0)
	{
	cPARAM = REMOVE_STRING(cDATATEXT,cPARAM_SEPARATOR,1)  
	cPARAM = LEFT_STRING(cPARAM, LENGTH_STRING(cPARAM)-1) 
	IF(nEscFound == 1)  { REPLACE_IN_STRING(cPARAM, $1B, cPARAM_SEPARATOR[1]) }
	RETURN cPARAM
	}
      ELSE
	{
	cPARAM = cDATATEXT
	IF(nEscFound == 1)  { REPLACE_IN_STRING(cPARAM, $1B, cPARAM_SEPARATOR[1]) }
	RETURN cPARAM
	}
      }
    ///////////////////////////////////////////////////////////////////////////
    // nRETURN_WHAT: Parameter 4:
    CASE 4:
      {
      // remove Command 
      REMOVE_STRING(cDATATEXT,cCMD_SEPARATOR,1)                                // INPUT-45,0,50    --> cCMD = INPUT-     --> cDATATEXT = 45,0,50

      // remove Parameter 1 
      REMOVE_STRING(cDATATEXT,cPARAM_SEPARATOR,1)                              // --> cDATATEXT = 0,50

      // remove Parameter 2 
      REMOVE_STRING(cDATATEXT,cPARAM_SEPARATOR,1)                              // --> cDATATEXT = 50   

      // remove Parameter 3 
      cRemString = REMOVE_STRING(cDATATEXT,cPARAM_SEPARATOR,1)                   
     
      IF(cRemString = '')
	{
	RETURN cPARAM
	}
     
      // extract Parameter 4 
      IF(FIND_STRING(cDATATEXT, cPARAM_SEPARATOR, 1) > 0) 
	{
	cPARAM = REMOVE_STRING(cDATATEXT,cPARAM_SEPARATOR,1) 
	cPARAM = LEFT_STRING(cPARAM, LENGTH_STRING(cPARAM)-1)
	IF(nEscFound == 1)  { REPLACE_IN_STRING(cPARAM, $1B, cPARAM_SEPARATOR[1]) }
	RETURN cPARAM
	}
      ELSE
	{
	cPARAM = cDATATEXT
	IF(nEscFound == 1)  { REPLACE_IN_STRING(cPARAM, $1B, cPARAM_SEPARATOR[1]) }
	RETURN cPARAM
	}
      }
    }//end SWITCH
    
  }
(*****************************************************************************)
(* End    DEFINE_FUNCTION  GET_FROM_DATATEXT                                 *)
(*****************************************************************************)
(************************************************************************)
(* START  DEFINE_FUNCTION  REPLACE_IN_STRING                            *)
(************************************************************************)
DEFINE_FUNCTION INTEGER REPLACE_IN_STRING(CHAR cSTRING[], CHAR cFindChar, CHAR nReplaceChar)
  {
  STACK_VAR INTEGER nStringLength
  STACK_VAR INTEGER nFoundCounter
  STACK_VAR INTEGER nRIS_ForLoop
  
  nFoundCounter = 0
  nStringLength = LENGTH_ARRAY(cSTRING)
  
  FOR(nRIS_ForLoop=1; nRIS_ForLoop<=nStringLength; nRIS_ForLoop++)
    {
    IF(cSTRING[nRIS_ForLoop] = cFindChar)
      {
      cSTRING[nRIS_ForLoop] = nReplaceChar
      nFoundCounter++
      }
    }
  
  RETURN nFoundCounter

  } 
(************************************************************************)
(* END    DEFINE_FUNCTION  REPLACE_IN_STRING	                        *)
(************************************************************************)
(************************************************************************)
(* START  DEFINE_FUNCTION  STRING_TO_DIAGNOSTICS 	                *)
(************************************************************************)
DEFINE_FUNCTION STRING_TO_DIAGNOSTICS(CHAR cOriginalString[2000], INTEGER nIndention, INTEGER nCallFromLine)
  {
  STACK_VAR CHAR cDivStringMem[300]
  STACK_VAR INTEGER nDivStringCount
  STACK_VAR CHAR cIndentionString[30]
  
  cOriginalString = "cOriginalString,'  (CFLN:',ITOA(nCallFromLine),')'"
  
  cIndentionString = FORMAT_STRING100('', nIndention, 'L', ' ') 
  
  IF(GET_LOG_LEVEL() <  AMX_DEBUG)
    {
    WAIT 600 'WAIT_STRING_TO_DIAGNOSTICS'
      {
      SEND_STRING 0,"'Message from STRING_TO_DIAGNOSTICS(): Strings are not visible in diagnostics. Log level is not set to AMX_DEBUG. Use SET_LOG_LEVEL() or telnet cmd ',$60,'netlinx log level',$60,' to change log level to AMX_DEBUG.'"
      }
    }
  ELSE
    {
    CANCEL_WAIT 'WAIT_STRING_TO_DIAGNOSTICS'
    }
  
  WHILE (LENGTH_ARRAY(cOriginalString)>150)
    {
    cDivStringMem=LEFT_STRING(cOriginalString,150)   
    REMOVE_STRING(cOriginalString,cDivStringMem,1)
    SELECT
      {
      ACTIVE(nDivStringCount = 0):  { AMX_LOG(AMX_DEBUG,cDivStringMem) }
      ACTIVE(1):                    { AMX_LOG(AMX_DEBUG,"cIndentionString,cDivStringMem") }
      }
    nDivStringCount++
    }
    
  SELECT
    {
    ACTIVE(nDivStringCount = 0):  { AMX_LOG(AMX_DEBUG,cOriginalString) }
    ACTIVE(1):                    { AMX_LOG(AMX_DEBUG,"cIndentionString,cOriginalString") }
    }
  }
(************************************************************************)
(* END    DEFINE_FUNCTION  STRING_TO_DIAGNOSTICS                        *)
(************************************************************************)
(*****************************************************************************)
(* START  DEFINE_FUNCTION  FORMAT_STRING100                                  *)
(*****************************************************************************)
DEFINE_FUNCTION CHAR[100] FORMAT_STRING100(CHAR cINPUT_STRING[100], INTEGER nTOTAL_LENGTH, CHAR cFILL_LEFT_OR_RIGHT, CHAR cFILL_WITH)
  {
  STACK_VAR INTEGER nINPUT_STRING_Length
  STACK_VAR INTEGER nFillLoop
  STACK_VAR CHAR cReturnString[100]
  
  SELECT
    {
    ACTIVE((nTOTAL_LENGTH < 1)  OR  (nTOTAL_LENGTH >100)):
      {
      RETURN 'ERR1'     // Invalid Parameter
      }
    ACTIVE( !(cFILL_LEFT_OR_RIGHT = 'L')  AND  !(cFILL_LEFT_OR_RIGHT = 'R')):
      {
      RETURN 'ERR2'     // Invalid Parameter
      }
    }
  
  nINPUT_STRING_Length = LENGTH_ARRAY(cINPUT_STRING)
  cReturnString = cINPUT_STRING
  
  SELECT
    {
    ACTIVE(nINPUT_STRING_Length > nTOTAL_LENGTH): 
      {
      cReturnString = LEFT_STRING(cINPUT_STRING,nTOTAL_LENGTH)
      RETURN cReturnString
      }
    }
  
  FOR(nFillLoop=nINPUT_STRING_Length; nFillLoop<nTOTAL_LENGTH; nFillLoop++)
    {
    SELECT
      {
      ACTIVE(cFILL_LEFT_OR_RIGHT = 'L'):
	 { 
	 cReturnString = "cFILL_WITH, cReturnString" 
	 }
      ACTIVE(cFILL_LEFT_OR_RIGHT = 'R'):
	 { 
	 cReturnString = "cReturnString, cFILL_WITH" 
	 }
      }
    }
  
  RETURN cReturnString
  }
(*****************************************************************************)
(* END    DEFINE_FUNCTION  FORMAT_STRING100                                  *)
(*****************************************************************************)
  
  
  
  
  
  
  
(***********************************************************)
(*                STARTUP CODE GOES BELOW                  *)
(***********************************************************)
DEFINE_START


// Timelines:
TIMELINE_CREATE(TL1_ID_FEEDBACK,   TL1_INTERVAL_FEEDBACK,   1, TIMELINE_RELATIVE, TIMELINE_REPEAT)
TIMELINE_CREATE(TL4_ID_TIMEOUT,    TL4_INTERVAL_TIMEOUT,    1, TIMELINE_RELATIVE, TIMELINE_REPEAT)

WAIT (dvSpkr___1.PORT%10)*10          // aviod synchronous polling
  {
  TIMELINE_CREATE(TL2_ID_TIMER1SEC,  TL2_INTERVAL_TIMER1SEC,  1, TIMELINE_RELATIVE, TIMELINE_REPEAT)
  }

sAuthorization.cType        = 'Basic'
SAuthorization.cCredentials = 'YWRtaW46YWRtaW4='   // = username 'admin' and password 'admin' coded in base64 


sSpkrStatus.fVolume = -200


// SET_LOG_LEVEL(AMX_DEBUG)           // for Debugging



(***********************************************************)
(*                THE EVENTS GO BELOW                      *)
(***********************************************************)
DEFINE_EVENT


(***********************************************************)
(* START  DATA_EVENT  vdvSpkr__1                           *)
(***********************************************************)
//
DATA_EVENT[vdvSpkr__1]                                                         // virtual device                                          
  {
  COMMAND:
    {
    STACK_VAR CHAR cDATATEXT[300]           
    STACK_VAR CHAR cCommand[30]
    
    STACK_VAR CHAR cParameter1[100]                                            // Parameter1 as CHAR
    STACK_VAR CHAR cParameter2[100]                                            // Parameter2 as CHAR
    
    STACK_VAR INTEGER nParameter1                                              // Parameter1 as INTEGER
    STACK_VAR INTEGER nParameter2                                              // Parameter2 as INTEGER
    
    STACK_VAR FLOAT fParameter1                                                // Parameter1 as FLOAT 
    STACK_VAR FLOAT fParameter2                                                // Parameter2 as FLOAT 
    
    cDATATEXT = DATA.TEXT                
    
    cCommand    = GET_FROM_DATATEXT("DATA.TEXT",0,'-',',')                     
    cParameter1 = GET_FROM_DATATEXT("DATA.TEXT",1,'-',',')                     // Parameter1 as CHAR
    cParameter2 = GET_FROM_DATATEXT("DATA.TEXT",2,'-',',')                     // Parameter2 as CHAR
    
    nParameter1 = ATOI(cParameter1)                                            // Parameter1 as INTEGER
    nParameter2 = ATOI(cParameter2)                                            // Parameter2 as INTEGER
    
    fParameter1 = ATOF(cParameter1)                                            // Parameter1 as FLOAT
    fParameter2 = ATOF(cParameter2)                                            // Parameter2 as FLOAT
    
    
    SWITCH(cCommand)
      {
      CASE 'POWER':                                                            // POWER-<state>       <state> = STANDBY | ACTIVE | BOOT
	{
	SELECT
	  {
	  ACTIVE((cParameter1 = 'STANDBY')  OR
	         (cParameter1 = 'ACTIVE') OR
	         (cParameter1 = 'BOOT')    ):
	    {
	    Send_PUT_method('/public/v1/device/pwr', "'{"state": "',cParameter1,'"}'")
            Send_GET_method('/public/v1/device/pwr')
	    }
	  ACTIVE(1):
	    {
	    SEND_STRING vdvSpkr__1,"'MSG_FROM_MODULE-ERROR,',cCommand,',invalid parameter,must be STANDBY|ACTIVE|BOOT'"
	    }
	  }
	}
      CASE 'PRESET':                                                           // PRESET-<preset>       <preset> = 0..5
	{
	SELECT
	  {
	  ACTIVE((nParameter1 >= 0)   AND
	         (nParameter1 <= 5)     ):
	    {
	    Send_PUT_method('/public/v1/profile/restore', "'{"id": ',ITOA(nParameter1),',"startup": false}'")
//	    Send_GET_method('/public/v1/profile/restore')            // no GET method available for profile/restore
	    CANCEL_WAIT 'profile_restore'
	    WAIT 10 'profile_restore'
	      {
	      Send_GET_method('/public/v1/audio/volume')
              Send_GET_method('/public/v1/audio/inputs')
	      Send_GET_method('/public/v1/profile/list')
	      }
	    }
	  ACTIVE(1):
	    {
	    SEND_STRING vdvSpkr__1,"'MSG_FROM_MODULE-ERROR,',cCommand,',invalid parameter,must be 0..5'"
	    }
	  }
	}
      CASE 'VOL_MUTE':                                                         // VOL_MUTE-<state>         <state> = TOGGLE | ON | OFF         
	{
	SWITCH(cParameter1)
	  {
	  CASE 'TOGGLE':
	    {
	    SELECT
	      {
	      ACTIVE(sSpkrStatus.cMute = 'false'):
		{
		Send_PUT_method('/public/v1/audio/volume', "'{"level": ',ITOA(sSpkrStatus.fVolume),',"mute": true}'")
		Send_GET_method('/public/v1/audio/volume')
		}
	      ACTIVE(1):
		{
		Send_PUT_method('/public/v1/audio/volume', "'{"level": ',ITOA(sSpkrStatus.fVolume),',"mute": false}'")
		Send_GET_method('/public/v1/audio/volume')
		}
	      }
	    }
	  CASE 'ON':
	    {
	    Send_PUT_method('/public/v1/audio/volume', "'{"level": ',ITOA(sSpkrStatus.fVolume),',"mute": true}'")
	    Send_GET_method('/public/v1/audio/volume')
	    }
	  CASE 'OFF':
	    {
	    Send_PUT_method('/public/v1/audio/volume', "'{"level": ',ITOA(sSpkrStatus.fVolume),',"mute": false}'")
	    Send_GET_method('/public/v1/audio/volume')
	    }
	  }
	}
      CASE 'VOL_LVL':                                                          // VOL_LVL-<vol>       <vol> = -200..0
	{
	SELECT
	  {
	  ACTIVE((fParameter1 >= -200)   AND
	         (fParameter1 <= 0)     ):
	    {
	    Send_PUT_method('/public/v1/audio/volume', "'{"level": ',FTOA(fParameter1),',"mute": ',sSpkrStatus.cMute,'}'")
	    Send_GET_method('/public/v1/audio/volume')
	    }
	  ACTIVE(1):
	    {
	    SEND_STRING vdvSpkr__1,"'MSG_FROM_MODULE-ERROR,',cCommand,',,invalid parameter,must be -200..0'"
	    }
	  }
	}
	CASE 'SET_VOLUME_STEPSIZE':                                            // SET_VOLUME_STEPSIZE-<small>,<large>   
	  {
	  SELECT
	    {
	    ACTIVE((fParameter1 <= 0)   OR
	           (fParameter1 > 20)    ):
	      {
	      SEND_STRING vdvSpkr__1,"'MSG_FROM_MODULE-ERROR,',cCommand,',invalid parameter 1,small step size must be > 0 and <= 20'"
	      }
	    ACTIVE((fParameter2 <= 0)   OR
	           (fParameter2 > 20)    ):
	      {
	      SEND_STRING vdvSpkr__1,"'MSG_FROM_MODULE-ERROR,',cCommand,',invalid parameter 2,large step size must be > 0 and <= 20'"
	      }
	    ACTIVE(fParameter1 > fParameter2):
	      {
	      SEND_STRING vdvSpkr__1,"'MSG_FROM_MODULE-ERROR,',cCommand,',invalid parameter,small step size must be <= large step size'"
	      }
	    ACTIVE(1):
	      {
              fVolStepSmall = fParameter1
              fVolStepLarge = fParameter2
	      }
	    }
	  }
	CASE 'SET_VOLUME_RANGE':                                               // SET_VOLUME_RANGE-<min>,<max>   
	  {
	  SELECT
	    {
	    ACTIVE((fParameter1 < -200)   OR
	           (fParameter1 >    0)    ):
	      {
	      SEND_STRING vdvSpkr__1,"'MSG_FROM_MODULE-ERROR,',cCommand,',invalid parameter 1,min must be -200..0'"
	      }
	    ACTIVE((fParameter2 < -200)   OR
	           (fParameter2 >    0)    ):
	      {
	      SEND_STRING vdvSpkr__1,"'MSG_FROM_MODULE-ERROR,',cCommand,',invalid parameter 2,max must be -200..0'"
	      }
	    ACTIVE(fParameter1 >= fParameter2):
	      {
	      SEND_STRING vdvSpkr__1,"'MSG_FROM_MODULE-ERROR,',cCommand,',invalid parameter,min must be < max'"
	      }
	    ACTIVE(1):
	      {
              fVolMin = fParameter1
              fVolMax = fParameter2
	      }
	    }
	  }
	  
	CASE 'GET_DEVICE_INFO':                                                // GET_DEVICE_INFO-<path>   
	  {
	  SELECT
	    {
	    ACTIVE(cParameter1 = '/device/pwr'):
	      {
	      SEND_STRING vdvSpkr__1,"'DEVICE_INFO-/device/pwr,state,',sSpkrStatus.cDevice_pwr_State"  
	      SEND_STRING vdvSpkr__1,"'DEVICE_INFO-/device/pwr,poeAllocatedPwr,',FTOA(sSpkrStatus.fDevice_pwr_PoeAllocatedPwr)"  
	      SEND_STRING vdvSpkr__1,"'DEVICE_INFO-/device/pwr,poePd15W,',ITOA(sSpkrStatus.nDevice_pwr_PoePd15W)"  
	      }
	    ACTIVE(cParameter1 = '/audio/inputs'):
	      {
	      SEND_STRING vdvSpkr__1,"'DEVICE_INFO-/audio/inputs,analog,',ITOA(sSpkrStatus.nInput_Analog)"  
	      SEND_STRING vdvSpkr__1,"'DEVICE_INFO-/audio/inputs,AoIP01,',ITOA(sSpkrStatus.nInput_AoIP01)"  
	      SEND_STRING vdvSpkr__1,"'DEVICE_INFO-/audio/inputs,AoIP02,',ITOA(sSpkrStatus.nInput_AoIP02)"  
	      }
	    ACTIVE(cParameter1 = '/audio/volume'):
	      {
	      SEND_STRING vdvSpkr__1,"'DEVICE_INFO-/audio/volume,level,',FTOA(sSpkrStatus.fVolume)"  
	      SEND_STRING vdvSpkr__1,"'DEVICE_INFO-/audio/volume,mute,', sSpkrStatus.cMute"  
	      }
	    ACTIVE(cParameter1 = '/network/zone'):
	      {
	      SEND_STRING vdvSpkr__1,"'DEVICE_INFO-/network/zone,zone,',ITOA(sSpkrStatus.nNetwork_zone_zone)"  
	      SEND_STRING vdvSpkr__1,"'DEVICE_INFO-/network/zone,name,', sSpkrStatus.cNetwork_zone_name"  
	      }
	    ACTIVE(cParameter1 = '/events'):
	      {
	      SEND_STRING vdvSpkr__1,"'DEVICE_INFO-/events,bsLevel,',FTOA(sSpkrStatus.fEvents_bsLevel)"  
	      SEND_STRING vdvSpkr__1,"'DEVICE_INFO-/events,twLevel,',FTOA(sSpkrStatus.fEvents_twLevel)"  
	      SEND_STRING vdvSpkr__1,"'DEVICE_INFO-/events,inLevel,',FTOA(sSpkrStatus.fEvents_inLevel)"  
	      SEND_STRING vdvSpkr__1,"'DEVICE_INFO-/events,cpuT,',FTOA(sSpkrStatus.fEvents_cpuT)"  
	      SEND_STRING vdvSpkr__1,"'DEVICE_INFO-/events,twCoilT,',FTOA(sSpkrStatus.fEvents_twCoilT)"  
	      SEND_STRING vdvSpkr__1,"'DEVICE_INFO-/events,nwInKbps,',FTOA(sSpkrStatus.fEvents_nwInKbps)"  
	      SEND_STRING vdvSpkr__1,"'DEVICE_INFO-/events,cpuLoad,',FTOA(sSpkrStatus.fEvents_cpuLoad)"  
	      SEND_STRING vdvSpkr__1,"'DEVICE_INFO-/events,uptime,',sSpkrStatus.cEvents_uptime"  
	      }
	    ACTIVE(cParameter1 = '/profile/list'):
	      {
	      SEND_STRING vdvSpkr__1,"'DEVICE_INFO-/profile/list,selected,',ITOA(sSpkrStatus.nProfile_list_selected)"  
	      SEND_STRING vdvSpkr__1,"'DEVICE_INFO-/profile/list,startup,',ITOA(sSpkrStatus.nProfile_list_startup)"  
	      SEND_STRING vdvSpkr__1,"'DEVICE_INFO-/profile/list,name/id0,',sSpkrStatus.cProfile_list_name[1]"  
	      SEND_STRING vdvSpkr__1,"'DEVICE_INFO-/profile/list,name/id1,',sSpkrStatus.cProfile_list_name[2]"  
	      SEND_STRING vdvSpkr__1,"'DEVICE_INFO-/profile/list,name/id2,',sSpkrStatus.cProfile_list_name[3]"  
	      SEND_STRING vdvSpkr__1,"'DEVICE_INFO-/profile/list,name/id3,',sSpkrStatus.cProfile_list_name[4]"  
	      SEND_STRING vdvSpkr__1,"'DEVICE_INFO-/profile/list,name/id4,',sSpkrStatus.cProfile_list_name[5]"  
	      SEND_STRING vdvSpkr__1,"'DEVICE_INFO-/profile/list,name/id5,',sSpkrStatus.cProfile_list_name[6]"  
	      }
	    ACTIVE(cParameter1 = '/device/id'):
	      {
	      SEND_STRING vdvSpkr__1,"'DEVICE_INFO-/device/id,barcode,',sSpkrStatus.cDevice_id_barcode"  
	      SEND_STRING vdvSpkr__1,"'DEVICE_INFO-/device/id,mac,',sSpkrStatus.cDevice_id_mac"  
	      SEND_STRING vdvSpkr__1,"'DEVICE_INFO-/device/id,hwId,',sSpkrStatus.cDevice_id_hwId"  
	      SEND_STRING vdvSpkr__1,"'DEVICE_INFO-/device/id,model,',sSpkrStatus.cDevice_id_model"  
	      SEND_STRING vdvSpkr__1,"'DEVICE_INFO-/device/id,modId,',sSpkrStatus.cDevice_id_modId"  
	      SEND_STRING vdvSpkr__1,"'DEVICE_INFO-/device/id,calibr,',sSpkrStatus.cDevice_id_calibr"  
	      }
	    ACTIVE(cParameter1 = '/device/info'):
	      {
	      SEND_STRING vdvSpkr__1,"'DEVICE_INFO-/device/info,fwId,',sSpkrStatus.cDevice_info_fwId"  
	      SEND_STRING vdvSpkr__1,"'DEVICE_INFO-/device/info,build,',sSpkrStatus.cDevice_info_build"  
	      SEND_STRING vdvSpkr__1,"'DEVICE_INFO-/device/info,baseId,',sSpkrStatus.cDevice_info_baseId"  
	      SEND_STRING vdvSpkr__1,"'DEVICE_INFO-/device/info,hwId,',sSpkrStatus.cDevice_info_hwId"  
	      SEND_STRING vdvSpkr__1,"'DEVICE_INFO-/device/info,model,',sSpkrStatus.cDevice_info_model"  
	      SEND_STRING vdvSpkr__1,"'DEVICE_INFO-/device/info,category,',sSpkrStatus.cDevice_info_category"  
	      SEND_STRING vdvSpkr__1,"'DEVICE_INFO-/device/info,technology,',sSpkrStatus.cDevice_info_technology"  
	      SEND_STRING vdvSpkr__1,"'DEVICE_INFO-/device/info,upgradeId,',sSpkrStatus.cDevice_info_upgradeId"  
	      SEND_STRING vdvSpkr__1,"'DEVICE_INFO-/device/info,apiVer,',sSpkrStatus.cDevice_info_apiVer"  
	      }
	    ACTIVE(1):
	      {
	      SEND_STRING vdvSpkr__1,"'MSG_FROM_MODULE-ERROR,',cCommand,',invalid parameter'"
	      }
	    }
	  }
	  
      }//end SWITCH(cCommand)
    }//end COMMAND:
  }
(***********************************************************)
(* END    DATA_EVENT  vdvSpkr__1                           *)
(***********************************************************)








(***********************************************************)
(* START  DATA_EVENT  dvSpkr___1                           *)
(***********************************************************)
//
DATA_EVENT[dvSpkr___1]                                                         // real device
  {
  ONLINE: 
    { 
    DEBUG("'---------- ONLINE'", __LINE__)
    sSpkrStatus.nIsOnline = 1
    nSPKR_send_ready = 1
    }
  OFFLINE: 
    { 
    DEBUG("'---------- OFFLINE'", __LINE__)
    sSpkrStatus.nIsOnline = 0
    CANCEL_WAIT 'send_ready'
    nSPKR_send_ready = 0
    }
  STRING:
    {
    cSPKR_RX_BUFFER = "cSPKR_RX_BUFFER,DATA.TEXT"
    IF( NOT(cSPKR_RX_BUFFER[LENGTH_ARRAY(cSPKR_RX_BUFFER)] = "$0A") )
      {
      cSPKR_RX_BUFFER = "cSPKR_RX_BUFFER,$0D,$0A"
      }
    
    IF(sSpkrStatus.nDEVICE_COMMUNICATING = 0)                                  // speaker has changed from offline to online
      {
      IF(nPollingPause < 99)
	{
	Send_GET_method('/public/v1/device/id')
	Send_GET_method('/public/v1/device/info')
	}
      }
      
    CANCEL_WAIT 'DEVICE_COMMUNICATING'
    sSpkrStatus.nDEVICE_COMMUNICATING = 1  
    WAIT 120 'DEVICE_COMMUNICATING'
      {
      sSpkrStatus.nDEVICE_COMMUNICATING = 0  
      }
      
    ReadRxBuffer()
    
    
    IF(TIMELINE_ACTIVE(TL4_ID_TIMEOUT))  
      {                       
      TIMELINE_KILL(TL4_ID_TIMEOUT)
      TIMELINE_CREATE(TL4_ID_TIMEOUT,  TL4_INTERVAL_TIMEOUT,   1, TIMELINE_RELATIVE, TIMELINE_REPEAT)
      }
    
    }
  ONERROR:  
    {
    DEBUG("'!!!!!!!! ONERROR ',ITOA(DATA.NUMBER)", __LINE__)
    if((data.number == 7) or (data.number == 17)) 
      {
      IP_CLIENT_OPEN(dvSpkr___1.port, cIpAddress, nSPKR_IP_PORT, IP_TCP) 
      DEBUG("'IP_CLIENT_OPEN'", __LINE__)
      }
    }
  }
(***********************************************************)
(* END    DATA_EVENT  dvSpkr___1                           *)
(***********************************************************)
(***********************************************************)
(* START  CHANNEL_EVENT  VOLUME UP/DOWN                    *)
(***********************************************************)
CHANNEL_EVENT[vdvSpkr__1,VOL_UP]                                               // VOL_UP  = 24    // Ramping:   Ramp volume up    (Small Step for BUTTON_EVENT-PUSH:)
  {
  ON:
    {
    STACK_VAR FLOAT fVolTempUp
    IF(nSPKR_send_ready = 1)
      {
      fVolTempUp = sSpkrStatus.fVolume + fVolStepSmall
      IF(fVolTempUp > fVolMax)
	{
	fVolTempUp = fVolMax
	}
      Send_PUT_method('/public/v1/audio/volume', "'{"level": ',FTOA(fVolTempUp),',"mute": ',sSpkrStatus.cMute,'}'")
      Send_GET_method('/public/v1/audio/volume')
      }
    }
  OFF: {}
  }
CHANNEL_EVENT[vdvSpkr__1,VOL_DN]                                               // VOL_DN =  25    // Ramping:   Ramp volume down    (Small Step for BUTTON_EVENT-PUSH:)
  {
  ON:
    {
    STACK_VAR FLOAT fVolTempDn
    IF(nSPKR_send_ready = 1)
      {
      fVolTempDn = sSpkrStatus.fVolume - fVolStepSmall
      IF(fVolTempDn < fVolMin)
	{
	fVolTempDn = fVolMin
	}
      Send_PUT_method('/public/v1/audio/volume', "'{"level": ',FTOA(fVolTempDn),',"mute": ',sSpkrStatus.cMute,'}'")
      Send_GET_method('/public/v1/audio/volume')
      }
    }
  OFF: {}
  }
CHANNEL_EVENT[vdvSpkr__1,GAIN_UP]                                              // GAIN_UP = 140   // Ramping:   Ramp gain up    (Large Step for BUTTON_EVENT-HOLD:)
  {
  ON:
    {
    STACK_VAR FLOAT fVolTempUp
    IF(nSPKR_send_ready = 1)
      {
      fVolTempUp = sSpkrStatus.fVolume + fVolStepLarge
      IF(fVolTempUp > fVolMax)
	{
	fVolTempUp = fVolMax
	}
      Send_PUT_method('/public/v1/audio/volume', "'{"level": ',FTOA(fVolTempUp),',"mute": ',sSpkrStatus.cMute,'}'")
      Send_GET_method('/public/v1/audio/volume')
      }
    }
  OFF: {}
  }
CHANNEL_EVENT[vdvSpkr__1,GAIN_DN]                                              // GAIN_DN = 141   // Ramping:   Ramp gain down    (Large Step for BUTTON_EVENT-HOLD:)
  {
  ON:
    {
    STACK_VAR FLOAT fVolTempDn
    IF(nSPKR_send_ready = 1)
      {
      fVolTempDn = sSpkrStatus.fVolume - fVolStepLarge
      IF(fVolTempDn < fVolMin)
	{
	fVolTempDn = fVolMin
	}
      Send_PUT_method('/public/v1/audio/volume', "'{"level": ',FTOA(fVolTempDn),',"mute": ',sSpkrStatus.cMute,'}'")
      Send_GET_method('/public/v1/audio/volume')
      }
    }
  OFF: {}
  }
(***********************************************************)
(* END    CHANNEL_EVENT  VOLUME UP/DOWN                    *)
(***********************************************************)

(***********************************************************)
(* START  CHANNEL_EVENT  Input Selector                    *)
(***********************************************************)
CHANNEL_EVENT[vdvSpkr__1,SOURCE_TV1]                                           // SOURCE_TV1 = 30   // Input on/off Toggle for Analog
CHANNEL_EVENT[vdvSpkr__1,SOURCE_VIDEO1]                                        // SOURCE_TV1 = 31   // Input on/off Toggle for AoIP input channel 1
CHANNEL_EVENT[vdvSpkr__1,SOURCE_VIDEO2]                                        // SOURCE_TV1 = 32   // Input on/off Toggle for AoIP input channel 2
  {
  ON:
    {
    STACK_VAR INTEGER nInput_Analog
    STACK_VAR INTEGER nInput_AoIP01
    STACK_VAR INTEGER nInput_AoIP02
    
    nInput_Analog = sSpkrStatus.nInput_Analog
    nInput_AoIP01 = sSpkrStatus.nInput_AoIP01
    nInput_AoIP02 = sSpkrStatus.nInput_AoIP02
    
    SWITCH(CHANNEL.CHANNEL)
      {
      CASE SOURCE_TV1:     { nInput_Analog =! nInput_Analog }
      CASE SOURCE_VIDEO1:  { nInput_AoIP01 =! nInput_AoIP01 }
      CASE SOURCE_VIDEO2:  { nInput_AoIP02 =! nInput_AoIP02 }
      }
      
    cSET_INPUT_SELECTOR_COMMAND = ''
    IF(nInput_Analog = 1)
      {
      cSET_INPUT_SELECTOR_COMMAND = '"A"'
      }
    IF(nInput_AoIP01 = 1)
      {
      IF(LENGTH_ARRAY(cSET_INPUT_SELECTOR_COMMAND) > 0)  { cSET_INPUT_SELECTOR_COMMAND = "cSET_INPUT_SELECTOR_COMMAND,','" }
      cSET_INPUT_SELECTOR_COMMAND = "cSET_INPUT_SELECTOR_COMMAND,'"AoIP01"'"
      }
    IF(nInput_AoIP02 = 1)
      {
      IF(LENGTH_ARRAY(cSET_INPUT_SELECTOR_COMMAND) > 0)  { cSET_INPUT_SELECTOR_COMMAND = "cSET_INPUT_SELECTOR_COMMAND,','" }
      cSET_INPUT_SELECTOR_COMMAND = "cSET_INPUT_SELECTOR_COMMAND,'"AoIP02"'"
      }
    
    CANCEL_WAIT 'SET_INPUT_SELECTOR'
    WAIT 5 'SET_INPUT_SELECTOR'                                                // wait to avoid sending multiple commands 
      {
      Send_PUT_method('/public/v1/audio/inputs', "'{"input": [',cSET_INPUT_SELECTOR_COMMAND,']}'")          // cSET_INPUT_SELECTOR_COMMAND: eg.  "A","AoIP01","AoIP02"
      Send_GET_method('/public/v1/audio/inputs')
      }
      
    }
  OFF: {}
  }
(***********************************************************)
(* END    CHANNEL_EVENT  Input Selector                    *)
(***********************************************************)


(************************************************************************)
(* Start  DEFINE_EVENT  Timelines                                       *)
(************************************************************************)

///////////////////////////////////////////////////////////////////////////////
// Timeline 1:
//
TIMELINE_EVENT[TL1_ID_FEEDBACK]                                                // Timeline-1: Feedback  100ms
  {
  
  
  [vdvSpkr__1,70]                    = (sSpkrStatus.nInput_Analog = 1)         //  70 = Feedback for Analog input        
  [vdvSpkr__1,71]                    = (sSpkrStatus.nInput_AoIP01 = 1)         //  71 = Feedback for AoIP input channel 1
  [vdvSpkr__1,72]                    = (sSpkrStatus.nInput_AoIP02 = 1)         //  72 = Feedback for AoIP input channel 2
  [vdvSpkr__1,VOL_MUTE_FB]           = (sSpkrStatus.cMute = 'true')            // 199 (VOL_MUTE_FB)
  [vdvSpkr__1,DEVICE_COMMUNICATING]  =  sSpkrStatus.nDEVICE_COMMUNICATING      // 251 (DEVICE_COMMUNICATING) 
  [vdvSpkr__1,POWER_FB]              =  sSpkrStatus.nPOWER_FB                  // 255 (POWER_FB) 
  
  
  IF( (FIND_STRING(cSPKR_TX_BUFFER, "$ff", 1))  AND         // wenn ein Command im TX Buffer ist
      (nSPKR_send_ready = 1       )               )         // wenn bereit zum Senden
    {
    SendFromTxBufferToDevice() 
    }
  
  }


///////////////////////////////////////////////////////////////////////////////
// Timeline 2:
//
TIMELINE_EVENT[TL2_ID_TIMER1SEC]
  {
  
  
  SELECT
    {
    ACTIVE(FIND_STRING(cSPKR_TX_BUFFER, "$ff", 1) > 0):        
      {
      // do nothing...
      }
    ACTIVE(nPollingPause > 0):                                 
      {
      nPollingPause--
      }
    ACTIVE(sSpkrStatus.nPOWER_FB = 0):                         // Device will not respond to all command during ISS_SLEEP and STANDBY state
      {                                                        // Instead of not responsing, the device goes offline 
      Send_GET_method('/public/v1/device/pwr')                 // The only command which works proper in STANDBY state is device/pwr
      }
    ACTIVE(TIMELINE.REPETITION % 24 = 0):  
      {
      Send_GET_method('/public/v1/device/pwr')
      }
    ACTIVE(TIMELINE.REPETITION % 24 = 3):   
      {
      Send_GET_method('/public/v1/audio/volume')
      }
    ACTIVE(TIMELINE.REPETITION % 24 = 7): 
      {
      Send_GET_method('/public/v1/audio/inputs')
      }
    ACTIVE(TIMELINE.REPETITION % 24 = 11): 
      {
      Send_GET_method('/public/v1/events')
      }
    ACTIVE(TIMELINE.REPETITION % 24 = 15): 
      {
      Send_GET_method('/public/v1/network/zone')
      }
    ACTIVE(TIMELINE.REPETITION % 24 = 19):   
      {
      Send_GET_method('/public/v1/profile/list')
      }
    }

  }//end Timeline
   
  

  
///////////////////////////////////////////////////////////////////////////////
// Timeline 4:
//
TIMELINE_EVENT[TL4_ID_TIMEOUT]
  {
  DEBUG("'waitConnectionTimeout'", __LINE__)
  cSPKR_TX_BUFFER = ''
  IF(sSpkrStatus.nIsOnline) 
    {
    IP_CLIENT_CLOSE(dvSpkr___1.port)
    DEBUG("'IP_CLIENT_CLOSE'", __LINE__)
    
    // STRING_TO_DIAGNOSTICS("'SPKR_DEBUG: TL4_ID_TIMEOUT: IP_CLIENT_CLOSE'", 12, __LINE__) 
    }
  nSPKR_send_ready = 1
  }
   


