MODULE_NAME='M_Genelec_44x0A_UI, REV04' (DEV vdvSpkr_all[],          // virtual device array      ---->>> Note: nSpeakerCount must be array length of vdvSpkr_all[]
					 DEV vdvTP____2)             // touchpanel 





(******************************************************************************************)
(* Start  History                                                                         *)
(******************************************************************************************)
(*                                                                                        *)
(* 17.09.2020/CL: Rev01 created                                                           *)
(* 25.09.2020/CL: Rev02 created                                                           *)
(* 05.10.2020/CL: Rev03 created                                                           *)
(* 04.03.2022/CL: Rev04 created                                                           *)
(*                                                                                        *)
(*                                                                                        *)
(******************************************************************************************)




(*******************************************************************)
(* Important information:                                          *)
(* ----------------------                                          *)
(* Running Genelec's Smart IP Manager Software                     *)
(* and                                                             *)
(* connecting to Speaker over API  (e.g. by this AMX Comm module)  *)
(* at the same time is NOT supported by Genelec.                   *)
(*******************************************************************)



(******************************************************************************************)
(* UI functionality                                                                       *)
(******************************************************************************************)
(*                                                                                        *)
(* Power On/Off                                  on Btn-Channel 01 + 02                   *)
(* Link for Vol+Mute                             on Btn-Channel 09                        *)
(*                                                                                        *)
(* Volume Up                      for 8 Speaker  on Btn-Channel 21,31,41...81             *)
(* Volume Down                    for 8 Speaker  on Btn-Channel 22,32,42...82             *)
(* Mute Toggle                    for 8 Speaker  on Btn-Channel 23,33,43...83             *)
(* -50dB                          for 8 Speaker  on Btn-Channel 24,34,44...84             *)
(*                                                                                        *)
(* Profile Recall              for Profile 0..5  on Btn-Channel 100...105                 *)
(*                                                                                        *)
(* POWER_FB-Feedback              for 8 Speaker  on Btn-Channel 231..238                  *)
(* DEVICE_COMMUNICATING-Feedback  for 8 Speaker  on Btn-Channel 241..248                  *)
(*                                                                                        *)
(* Note: nSpeakerCount must be array length of vdvSpkr_all[]   !!                         *)
(*                                                                                        *)
(******************************************************************************************)






(***********************************************************)
(*          DEVICE NUMBER DEFINITIONS GO BELOW             *)
(***********************************************************)
DEFINE_DEVICE





// INCLUDE's:
#INCLUDE 'SNAPI.axi'


(***********************************************************)
(*               CONSTANT DEFINITIONS GO BELOW             *)
(***********************************************************)
DEFINE_CONSTANT

// Timeline Identifier:
INTEGER TL2_ID_FEEDBACK       = 2                         


INTEGER nSpeakerCount = 2                                                      // Note: nSpeakerCount must be array length of vdvSpkr_all[]








(***********************************************************)
(*               VARIABLE DEFINITIONS GO BELOW             *)
(***********************************************************)
DEFINE_VARIABLE


// Timelines Interval:
VOLATILE LONG TL2_INTERVAL_FEEDBACK[]            =   {200}                     // 0.2 sec     Interval for timeline


VOLATILE INTEGER nSELECTED_INPUT_FB[nSpeakerCount][3]                          // Feedback for selected input
                                                                               // [index2] : [x][1] = vdv Channel 70 = Feedback for Analog input        
                                                                               //          : [x][2] = vdv Channel 71 = Feedback for AoIP input channel 1
                                                                               //          : [x][3] = vdv Channel 72 = Feedback for AoIP input channel 2

VOLATILE INTEGER nVOL_MUTE_FB[nSpeakerCount]                                   // VOL_MUTE_FB,          vdv Channel 199
VOLATILE INTEGER nDEVICE_COMMUNICATING[nSpeakerCount]                          // DEVICE_COMMUNICATING, vdv Channel 251
VOLATILE INTEGER nPOWER_FB[nSpeakerCount]                                      // POWER_FB,             vdv Channel 255
VOLATILE FLOAT   fVolumeFeedback[nSpeakerCount]                                // Volume value from Comm-Module   -200..0

VOLATILE INTEGER nLink

VOLATILE INTEGER nProfileSelected = 99                                         // 0...5 
                                                                               // 99 = undefined






(***********************************************************)
(*        SUBROUTINE/FUNCTION DEFINITIONS GO BELOW         *)
(***********************************************************)

(*****************************************************************************)
(* START  DEFINE_FUNCTION  ClearInfoPopup                                    *)
(*****************************************************************************)
DEFINE_FUNCTION ClearInfoPopup()
  {
  SEND_COMMAND vdvTP____2,"'^TXT-120,1&2,'" 
  SEND_COMMAND vdvTP____2,"'^TXT-121,1&2,'" 
  SEND_COMMAND vdvTP____2,"'^TXT-122,1&2,'" 
  SEND_COMMAND vdvTP____2,"'^TXT-130,1&2,'" 
  SEND_COMMAND vdvTP____2,"'^TXT-131,1&2,'" 
  SEND_COMMAND vdvTP____2,"'^TXT-132,1&2,'" 
  SEND_COMMAND vdvTP____2,"'^TXT-140,1&2,'" 
  SEND_COMMAND vdvTP____2,"'^TXT-141,1&2,'" 
  SEND_COMMAND vdvTP____2,"'^TXT-150,1&2,'" 
  SEND_COMMAND vdvTP____2,"'^TXT-151,1&2,'" 
  SEND_COMMAND vdvTP____2,"'^TXT-160,1&2,'" 
  SEND_COMMAND vdvTP____2,"'^TXT-161,1&2,'" 
  SEND_COMMAND vdvTP____2,"'^TXT-162,1&2,'" 
  SEND_COMMAND vdvTP____2,"'^TXT-163,1&2,'" 
  SEND_COMMAND vdvTP____2,"'^TXT-164,1&2,'" 
  SEND_COMMAND vdvTP____2,"'^TXT-165,1&2,'" 
  SEND_COMMAND vdvTP____2,"'^TXT-166,1&2,'" 
  SEND_COMMAND vdvTP____2,"'^TXT-167,1&2,'" 
  SEND_COMMAND vdvTP____2,"'^TXT-170,1&2,'" 
  SEND_COMMAND vdvTP____2,"'^TXT-171,1&2,'" 
  SEND_COMMAND vdvTP____2,"'^TXT-172,1&2,'" 
  SEND_COMMAND vdvTP____2,"'^TXT-173,1&2,'" 
  SEND_COMMAND vdvTP____2,"'^TXT-174,1&2,'" 
  SEND_COMMAND vdvTP____2,"'^TXT-175,1&2,'" 
  SEND_COMMAND vdvTP____2,"'^TXT-176,1&2,'" 
  SEND_COMMAND vdvTP____2,"'^TXT-177,1&2,'" 
  SEND_COMMAND vdvTP____2,"'^TXT-180,1&2,'" 
  SEND_COMMAND vdvTP____2,"'^TXT-181,1&2,'" 
  SEND_COMMAND vdvTP____2,"'^TXT-182,1&2,'" 
  SEND_COMMAND vdvTP____2,"'^TXT-183,1&2,'" 
  SEND_COMMAND vdvTP____2,"'^TXT-184,1&2,'" 
  SEND_COMMAND vdvTP____2,"'^TXT-185,1&2,'" 
  SEND_COMMAND vdvTP____2,"'^TXT-190,1&2,'" 
  SEND_COMMAND vdvTP____2,"'^TXT-191,1&2,'" 
  SEND_COMMAND vdvTP____2,"'^TXT-192,1&2,'" 
  SEND_COMMAND vdvTP____2,"'^TXT-193,1&2,'" 
  SEND_COMMAND vdvTP____2,"'^TXT-194,1&2,'" 
  SEND_COMMAND vdvTP____2,"'^TXT-195,1&2,'" 
  SEND_COMMAND vdvTP____2,"'^TXT-196,1&2,'" 
  SEND_COMMAND vdvTP____2,"'^TXT-197,1&2,'" 
  SEND_COMMAND vdvTP____2,"'^TXT-198,1&2,'" 
  }
(*****************************************************************************)
(* End    DEFINE_FUNCTION  ClearInfoPopup                                    *)
(*****************************************************************************)
(*****************************************************************************)
(* START  DEFINE_FUNCTION  GET_FROM_DATATEXT                                 *)
(*****************************************************************************)
// REV10 (version for 4 parameters)
//
DEFINE_FUNCTION CHAR[100] GET_FROM_DATATEXT(CHAR cDATATEXT[], INTEGER nRETURN_WHAT, CHAR cCMD_SEPARATOR[], CHAR cPARAM_SEPARATOR[])
  {
  STACK_VAR CHAR cCMD[30]    
  STACK_VAR CHAR cPARAM[100] 
  STACK_VAR CHAR cRemString[100]
  STACK_VAR INTEGER nEscPos                        
  STACK_VAR INTEGER nEscFound
  
  cCMD   = ''
  cPARAM = ''
  cRemString = ''
  nEscFound = 0
  
  // 
  nEscPos = FIND_STRING(cDATATEXT, "$1B,cPARAM_SEPARATOR", 1) 
  WHILE(nEscPos > 0)                                                           // nEscPos = 16
    {
    nEscFound = 1
    cDATATEXT = "LEFT_STRING(cDATATEXT,nEscPos),RIGHT_STRING(cDATATEXT, LENGTH_ARRAY(cDATATEXT)-nEscPos-1)" 
    nEscPos = FIND_STRING(cDATATEXT, "$1B,cPARAM_SEPARATOR", 1)
    }
  
  
  SWITCH(nRETURN_WHAT)
    {
    ///////////////////////////////////////////////////////////////////////////
    // nRETURN_WHAT: Command Keyword:
    CASE 0:
      {
      // extract Command 
      IF(FIND_STRING(cDATATEXT, cCMD_SEPARATOR, 1) = 0) 
	{
	RETURN cDATATEXT
	}
      
      cCMD = REMOVE_STRING(cDATATEXT,cCMD_SEPARATOR,1)                         // INPUT-45,0,50    --> cCMD = INPUT-     --> cDATATEXT = 45,0,50
      cCMD = LEFT_STRING(cCMD, LENGTH_STRING(cCMD)-1)
      RETURN cCMD
      }
    ///////////////////////////////////////////////////////////////////////////
    // nRETURN_WHAT: Parameter 1:
    CASE 1:
      {
      // remove Command
      cRemString = REMOVE_STRING(cDATATEXT,cCMD_SEPARATOR,1)                   // INPUT-45,0,50    --> cCMD = INPUT-     --> cDATATEXT = 45,0,50
     
      // 
      IF(cRemString = '')
	{
	RETURN cPARAM
	}
     
      // extract Parameter 1
      IF(FIND_STRING(cDATATEXT, cPARAM_SEPARATOR, 1) > 0)
	{
	cPARAM = REMOVE_STRING(cDATATEXT,cPARAM_SEPARATOR,1)                   // cDATATEXT = 45,0,50   --> cPARAM1 = 45,    --> cDATATEXT = 0,50    
	cPARAM = LEFT_STRING(cPARAM, LENGTH_STRING(cPARAM)-1)
	IF(nEscFound == 1)  { REPLACE_IN_STRING(cPARAM, $1B, cPARAM_SEPARATOR[1]) }
	RETURN cPARAM
	}
      ELSE
	{
	cPARAM = cDATATEXT  
	IF(nEscFound == 1)  { REPLACE_IN_STRING(cPARAM, $1B, cPARAM_SEPARATOR[1]) }
	RETURN cPARAM
	}
      }
    ///////////////////////////////////////////////////////////////////////////
    // nRETURN_WHAT: Parameter 2:
    CASE 2:
      {
      // remove Command
      REMOVE_STRING(cDATATEXT,cCMD_SEPARATOR,1)                                // INPUT-45,0,50    --> cCMD = INPUT-     --> cDATATEXT = 45,0,50                     

      // remove Parameter 1
      cRemString = REMOVE_STRING(cDATATEXT,cPARAM_SEPARATOR,1)                 // --> cDATATEXT = 0,50                             

      // 
      IF(cRemString = '')
	{
	RETURN cPARAM
	}
     
      // extract Parameter 2 
      IF(FIND_STRING(cDATATEXT, cPARAM_SEPARATOR, 1) > 0)
	{
	cPARAM = REMOVE_STRING(cDATATEXT,cPARAM_SEPARATOR,1)                   // cDATATEXT = 45,0,50   --> cPARAM1 = 45,    --> cDATATEXT = 0,50    
	cPARAM = LEFT_STRING(cPARAM, LENGTH_STRING(cPARAM)-1)
	IF(nEscFound == 1)  { REPLACE_IN_STRING(cPARAM, $1B, cPARAM_SEPARATOR[1]) }
	RETURN cPARAM
	}
      ELSE
	{
	cPARAM = cDATATEXT 
	IF(nEscFound == 1)  { REPLACE_IN_STRING(cPARAM, $1B, cPARAM_SEPARATOR[1]) }
	RETURN cPARAM
	}
      }
    ///////////////////////////////////////////////////////////////////////////
    // nRETURN_WHAT: Parameter 3:
    CASE 3:
      {
      // remove Command
      REMOVE_STRING(cDATATEXT,cCMD_SEPARATOR,1)                                // INPUT-45,0,50    --> cCMD = INPUT-     --> cDATATEXT = 45,0,50 

      // remove Parameter 1
      REMOVE_STRING(cDATATEXT,cPARAM_SEPARATOR,1)                              // --> cDATATEXT = 0,50            

      // remove Parameter 2
      cRemString = REMOVE_STRING(cDATATEXT,cPARAM_SEPARATOR,1)                 // --> cDATATEXT = 50     
     
      //
      IF(cRemString = '')
	{
	RETURN cPARAM
	}
     
      // extract Parameter 3 
      IF(FIND_STRING(cDATATEXT, cPARAM_SEPARATOR, 1) > 0)
	{
	cPARAM = REMOVE_STRING(cDATATEXT,cPARAM_SEPARATOR,1)  
	cPARAM = LEFT_STRING(cPARAM, LENGTH_STRING(cPARAM)-1) 
	IF(nEscFound == 1)  { REPLACE_IN_STRING(cPARAM, $1B, cPARAM_SEPARATOR[1]) }
	RETURN cPARAM
	}
      ELSE
	{
	cPARAM = cDATATEXT
	IF(nEscFound == 1)  { REPLACE_IN_STRING(cPARAM, $1B, cPARAM_SEPARATOR[1]) }
	RETURN cPARAM
	}
      }
    ///////////////////////////////////////////////////////////////////////////
    // nRETURN_WHAT: Parameter 4:
    CASE 4:
      {
      // remove Command 
      REMOVE_STRING(cDATATEXT,cCMD_SEPARATOR,1)                                // INPUT-45,0,50    --> cCMD = INPUT-     --> cDATATEXT = 45,0,50

      // remove Parameter 1 
      REMOVE_STRING(cDATATEXT,cPARAM_SEPARATOR,1)                              // --> cDATATEXT = 0,50

      // remove Parameter 2 
      REMOVE_STRING(cDATATEXT,cPARAM_SEPARATOR,1)                              // --> cDATATEXT = 50   

      // remove Parameter 3 
      cRemString = REMOVE_STRING(cDATATEXT,cPARAM_SEPARATOR,1)                   
     
      IF(cRemString = '')
	{
	RETURN cPARAM
	}
     
      // extract Parameter 4 
      IF(FIND_STRING(cDATATEXT, cPARAM_SEPARATOR, 1) > 0) 
	{
	cPARAM = REMOVE_STRING(cDATATEXT,cPARAM_SEPARATOR,1) 
	cPARAM = LEFT_STRING(cPARAM, LENGTH_STRING(cPARAM)-1)
	IF(nEscFound == 1)  { REPLACE_IN_STRING(cPARAM, $1B, cPARAM_SEPARATOR[1]) }
	RETURN cPARAM
	}
      ELSE
	{
	cPARAM = cDATATEXT
	IF(nEscFound == 1)  { REPLACE_IN_STRING(cPARAM, $1B, cPARAM_SEPARATOR[1]) }
	RETURN cPARAM
	}
      }
    }//end SWITCH
    
  }
(*****************************************************************************)
(* End    DEFINE_FUNCTION  GET_FROM_DATATEXT                                 *)
(*****************************************************************************)
(*****************************************************************************)
(* BEGIN DEFINE_FUNCTION  REPLACE_IN_STRING                                  *)
(*****************************************************************************)
DEFINE_FUNCTION INTEGER REPLACE_IN_STRING(CHAR cSTRING[], CHAR cFindChar, CHAR nReplaceChar)
  {
  STACK_VAR INTEGER nStringLength
  STACK_VAR INTEGER nFoundCounter
  STACK_VAR INTEGER nRIS_ForLoop
  
  nFoundCounter = 0
  nStringLength = LENGTH_ARRAY(cSTRING)
  
  FOR(nRIS_ForLoop=1; nRIS_ForLoop<=nStringLength; nRIS_ForLoop++)
    {
    IF(cSTRING[nRIS_ForLoop] = cFindChar)
      {
      cSTRING[nRIS_ForLoop] = nReplaceChar
      nFoundCounter++
      }
    }
  
  RETURN nFoundCounter
  } 
(*****************************************************************************)
(* END   DEFINE_FUNCTION  REPLACE_IN_STRING	                             *)
(*****************************************************************************)




(***********************************************************)
(*                STARTUP CODE GOES BELOW                  *)
(***********************************************************)
DEFINE_START

// Timelines:
TIMELINE_CREATE(TL2_ID_FEEDBACK,   TL2_INTERVAL_FEEDBACK,   1, TIMELINE_RELATIVE, TIMELINE_REPEAT)





(***********************************************************)
(*                THE EVENTS GO BELOW                      *)
(***********************************************************)
DEFINE_EVENT



///////////////////////////////////////////////////////////////////////////////
// Communication from COMM-Module:
//

DATA_EVENT[vdvSpkr_all]                                                        // virtual devices of all speakers
  {
  STRING:
    {
    STACK_VAR INTEGER nVdvSpkr_all_Index
    
    STACK_VAR CHAR cDATATEXT[300]           
    STACK_VAR CHAR cCommand[30]
    
    STACK_VAR CHAR cParameter1[100]                                            // Parameter1 as CHAR
    STACK_VAR CHAR cParameter2[100]                                            // Parameter2 as CHAR
    STACK_VAR CHAR cParameter3[100]                                            // Parameter3 as CHAR
    
    cDATATEXT = DATA.TEXT                
    
    cCommand    = GET_FROM_DATATEXT("DATA.TEXT",0,'-',',')                     
    cParameter1 = GET_FROM_DATATEXT("DATA.TEXT",1,'-',',')                     // Parameter1 as CHAR
    cParameter2 = GET_FROM_DATATEXT("DATA.TEXT",2,'-',',')                     // Parameter2 as CHAR
    cParameter3 = GET_FROM_DATATEXT("DATA.TEXT",3,'-',',')                     // Parameter3 as CHAR
    
    nVdvSpkr_all_Index = GET_LAST(vdvSpkr_all)
    
    
    SWITCH(cCommand)
      {
      CASE 'DEVICE_INFO':                                                      // DEVICE_INFO-<path>,<property name>,<property value>
	{
	SWITCH(cParameter1)   // <path>
	  {
	  CASE '/device/pwr':
	    {
	    SWITCH(cParameter2)   // <property name>
	      {
	      CASE 'state':            { SEND_COMMAND vdvTP____2,"'^TXT-120,1&2,',cParameter2,' = ',cParameter3" }
	      CASE 'poeAllocatedPwr':  { SEND_COMMAND vdvTP____2,"'^TXT-121,1&2,',cParameter2,' = ',cParameter3" }
	      CASE 'poePd15W':         { SEND_COMMAND vdvTP____2,"'^TXT-122,1&2,',cParameter2,' = ',cParameter3" }
	      }
	    }
	  CASE '/audio/inputs':
	    {
	    SWITCH(cParameter2)   // <property name>
	      {
	      CASE 'analog':         { SEND_COMMAND vdvTP____2,"'^TXT-130,1&2,',cParameter2,' = ',cParameter3" }
	      CASE 'AoIP01':         { SEND_COMMAND vdvTP____2,"'^TXT-131,1&2,',cParameter2,' = ',cParameter3" }
	      CASE 'AoIP02':         { SEND_COMMAND vdvTP____2,"'^TXT-132,1&2,',cParameter2,' = ',cParameter3" }
	      }
	    }
	  CASE '/audio/volume':
	    {
	    SWITCH(cParameter2)   // <property name>
	      {
	      CASE 'level':         { SEND_COMMAND vdvTP____2,"'^TXT-140,1&2,',cParameter2,' = ',cParameter3" }
	      CASE 'mute':          { SEND_COMMAND vdvTP____2,"'^TXT-141,1&2,',cParameter2,' = ',cParameter3" }
	      }
	    }
	  CASE '/network/zone':
	    {
	    SWITCH(cParameter2)   // <property name>
	      {
	      CASE 'zone':          { SEND_COMMAND vdvTP____2,"'^TXT-150,1&2,',cParameter2,' = ',cParameter3" }
	      CASE 'name':          { SEND_COMMAND vdvTP____2,"'^TXT-151,1&2,',cParameter2,' = ',cParameter3" }
	      }
	    }
	  CASE '/events':
	    {
	    SWITCH(cParameter2)   // <property name>
	      {
	      CASE 'bsLevel':        { SEND_COMMAND vdvTP____2,"'^TXT-160,1&2,',cParameter2,' = ',cParameter3" }
	      CASE 'twLevel':        { SEND_COMMAND vdvTP____2,"'^TXT-161,1&2,',cParameter2,' = ',cParameter3" }
	      CASE 'inLevel':        { SEND_COMMAND vdvTP____2,"'^TXT-162,1&2,',cParameter2,' = ',cParameter3" }
	      CASE 'cpuT':           { SEND_COMMAND vdvTP____2,"'^TXT-163,1&2,',cParameter2,' = ',cParameter3" }
	      CASE 'twCoilT':        { SEND_COMMAND vdvTP____2,"'^TXT-164,1&2,',cParameter2,' = ',cParameter3" }
	      CASE 'nwInKbps':       { SEND_COMMAND vdvTP____2,"'^TXT-165,1&2,',cParameter2,' = ',cParameter3" }
	      CASE 'cpuLoad':        { SEND_COMMAND vdvTP____2,"'^TXT-166,1&2,',cParameter2,' = ',cParameter3" }
	      CASE 'uptime':         { SEND_COMMAND vdvTP____2,"'^TXT-167,1&2,',cParameter2,' = ',cParameter3" }
	      }
	    }
	  CASE '/profile/list':
	    {
	    SWITCH(cParameter2)   // <property name>
	      {
	      CASE 'selected':        { SEND_COMMAND vdvTP____2,"'^TXT-170,1&2,',cParameter2,' = ',cParameter3" 
	                                IF(nVdvSpkr_all_Index = 1)  { nProfileSelected = ATOI(cParameter3)} }
	      CASE 'startup':         { SEND_COMMAND vdvTP____2,"'^TXT-171,1&2,',cParameter2,' = ',cParameter3" }
	      CASE 'name/id0':        { SEND_COMMAND vdvTP____2,"'^TXT-172,1&2,',cParameter2,' = ',cParameter3" 
					SEND_COMMAND vdvTP____2,"'^TXT-100,1&2,',cParameter3"}
	      CASE 'name/id1':        { SEND_COMMAND vdvTP____2,"'^TXT-173,1&2,',cParameter2,' = ',cParameter3" 
					SEND_COMMAND vdvTP____2,"'^TXT-101,1&2,',cParameter3"}
	      CASE 'name/id2':        { SEND_COMMAND vdvTP____2,"'^TXT-174,1&2,',cParameter2,' = ',cParameter3" 
					SEND_COMMAND vdvTP____2,"'^TXT-102,1&2,',cParameter3"}
	      CASE 'name/id3':        { SEND_COMMAND vdvTP____2,"'^TXT-175,1&2,',cParameter2,' = ',cParameter3" 
					SEND_COMMAND vdvTP____2,"'^TXT-103,1&2,',cParameter3"}
	      CASE 'name/id4':        { SEND_COMMAND vdvTP____2,"'^TXT-176,1&2,',cParameter2,' = ',cParameter3" 
					SEND_COMMAND vdvTP____2,"'^TXT-104,1&2,',cParameter3"}
	      CASE 'name/id5':        { SEND_COMMAND vdvTP____2,"'^TXT-177,1&2,',cParameter2,' = ',cParameter3" 
					SEND_COMMAND vdvTP____2,"'^TXT-105,1&2,',cParameter3"}
	      }
	    }
	  CASE '/device/id':
	    {
	    SWITCH(cParameter2)   // <property name>
	      {
	      CASE 'barcode':       { SEND_COMMAND vdvTP____2,"'^TXT-180,1&2,',cParameter2,' = ',cParameter3" }
	      CASE 'mac':           { SEND_COMMAND vdvTP____2,"'^TXT-181,1&2,',cParameter2,' = ',cParameter3" }
	      CASE 'hwId':          { SEND_COMMAND vdvTP____2,"'^TXT-182,1&2,',cParameter2,' = ',cParameter3" }
	      CASE 'model':         { SEND_COMMAND vdvTP____2,"'^TXT-183,1&2,',cParameter2,' = ',cParameter3" }
	      CASE 'modId':         { SEND_COMMAND vdvTP____2,"'^TXT-184,1&2,',cParameter2,' = ',cParameter3" }
	      CASE 'calibr':        { SEND_COMMAND vdvTP____2,"'^TXT-185,1&2,',cParameter2,' = ',cParameter3" }
	      }
	    }
	  CASE '/device/info':
	    {
	    SWITCH(cParameter2)   // <property name>
	      {
	      CASE 'fwId':             { SEND_COMMAND vdvTP____2,"'^TXT-190,1&2,',cParameter2,' = ',cParameter3" }
	      CASE 'build':            { SEND_COMMAND vdvTP____2,"'^TXT-191,1&2,',cParameter2,' = ',cParameter3" }
	      CASE 'baseId':           { SEND_COMMAND vdvTP____2,"'^TXT-192,1&2,',cParameter2,' = ',cParameter3" }
	      CASE 'hwId':             { SEND_COMMAND vdvTP____2,"'^TXT-193,1&2,',cParameter2,' = ',cParameter3" }
	      CASE 'model':            { SEND_COMMAND vdvTP____2,"'^TXT-194,1&2,',cParameter2,' = ',cParameter3" }
	      CASE 'category':         { SEND_COMMAND vdvTP____2,"'^TXT-195,1&2,',cParameter2,' = ',cParameter3" }
	      CASE 'technology':       { SEND_COMMAND vdvTP____2,"'^TXT-196,1&2,',cParameter2,' = ',cParameter3" }
	      CASE 'upgradeId':        { SEND_COMMAND vdvTP____2,"'^TXT-197,1&2,',cParameter2,' = ',cParameter3" }
	      CASE 'apiVer':           { SEND_COMMAND vdvTP____2,"'^TXT-198,1&2,',cParameter2,' = ',cParameter3" }
	      }
	    }
	  }
	}
      }
    }
  }




// Volume change from COMM module:
LEVEL_EVENT[vdvSpkr_all,VOL_LVL]                                               // Level 1 = VOL_LVL
  {
  STACK_VAR INTEGER nVdvSpkr_all_Index
  nVdvSpkr_all_Index = GET_LAST(vdvSpkr_all)
  
  fVolumeFeedback[nVdvSpkr_all_Index] = LEVEL.VALUE/10
  SEND_LEVEL vdvTP____2, nVdvSpkr_all_Index, fVolumeFeedback[nVdvSpkr_all_Index]
  }



// Input State from COMM module:
CHANNEL_EVENT[vdvSpkr_all,70]                                                  // Channel 70 = Feedback for Analog input        
CHANNEL_EVENT[vdvSpkr_all,71]                                                  // Channel 71 = Feedback for AoIP input channel 1
CHANNEL_EVENT[vdvSpkr_all,72]                                                  // Channel 72 = Feedback for AoIP input channel 2
  {
  ON:
    {
    STACK_VAR INTEGER nVdvSpkr_all_Index
    nVdvSpkr_all_Index = GET_LAST(vdvSpkr_all)
    nSELECTED_INPUT_FB[nVdvSpkr_all_Index][CHANNEL.CHANNEL-69] = 1
    }
  OFF:
    {
    STACK_VAR INTEGER nVdvSpkr_all_Index
    nVdvSpkr_all_Index = GET_LAST(vdvSpkr_all)
    nSELECTED_INPUT_FB[nVdvSpkr_all_Index][CHANNEL.CHANNEL-69] = 0
    }
  }





// Mute change from COMM module:
CHANNEL_EVENT[vdvSpkr_all,VOL_MUTE_FB]                                         // Channel 199  = VOL_MUTE_FB 
  {
  ON:
    {
    STACK_VAR INTEGER nVdvSpkr_all_Index
    nVdvSpkr_all_Index = GET_LAST(vdvSpkr_all)
    nVOL_MUTE_FB[nVdvSpkr_all_Index] = 1
    }
  OFF:
    {
    STACK_VAR INTEGER nVdvSpkr_all_Index
    nVdvSpkr_all_Index = GET_LAST(vdvSpkr_all)
    nVOL_MUTE_FB[nVdvSpkr_all_Index] = 0
    }
  }
  
  
// DEVICE_COMMUNICATING from COMM module:
CHANNEL_EVENT[vdvSpkr_all,DEVICE_COMMUNICATING]                                // Channel 251  = DEVICE_COMMUNICATING 
  {
  ON:
    {
    STACK_VAR INTEGER nVdvSpkr_all_Index
    nVdvSpkr_all_Index = GET_LAST(vdvSpkr_all)
    nDEVICE_COMMUNICATING[nVdvSpkr_all_Index] = 1
    }
  OFF:
    {
    STACK_VAR INTEGER nVdvSpkr_all_Index
    nVdvSpkr_all_Index = GET_LAST(vdvSpkr_all)
    nDEVICE_COMMUNICATING[nVdvSpkr_all_Index] = 0
    }
  }
  
  
// POWER_FB from COMM module:
CHANNEL_EVENT[vdvSpkr_all,POWER_FB]                                            // Channel 255  = POWER_FB 
  {
  ON:
    {
    STACK_VAR INTEGER nVdvSpkr_all_Index
    nVdvSpkr_all_Index = GET_LAST(vdvSpkr_all)
    nPOWER_FB[nVdvSpkr_all_Index] = 1
    }
  OFF:
    {
    STACK_VAR INTEGER nVdvSpkr_all_Index
    nVdvSpkr_all_Index = GET_LAST(vdvSpkr_all)
    nPOWER_FB[nVdvSpkr_all_Index] = 0
    }
  }





///////////////////////////////////////////////////////////////////////////////
// Touchpanel:

BUTTON_EVENT[vdvTP____2,001]                                                   // Power ACTIVE
BUTTON_EVENT[vdvTP____2,002]                                                   // Power STANDBY
  {
  PUSH:
    {
    TO[BUTTON.INPUT]
    
    SWITCH(BUTTON.INPUT.CHANNEL)
      {
      CASE 001:
	{
        SEND_COMMAND vdvSpkr_all, "'POWER-ACTIVE'"  
	}
      CASE 002:
	{
        SEND_COMMAND vdvSpkr_all, "'POWER-STANDBY'" 
	}
      }
    }
  RELEASE: { }
  }




BUTTON_EVENT[vdvTP____2,009]                                                   // Link
  {
  PUSH:
    {
    nLink =! nLink
    
    IF(nLink)        // if Link is activated, Speaker 2 get same values as Speaker 1 
      {
      SWITCH(nVOL_MUTE_FB[1])
	{
	CASE 0:   { SEND_COMMAND vdvSpkr_all[2], "'VOL_MUTE-OFF'" }
	CASE 1:   { SEND_COMMAND vdvSpkr_all[2], "'VOL_MUTE-ON'"  }
	}
      WAIT 10    
	{        
	SEND_COMMAND vdvSpkr_all[2], "'VOL_LVL-',FTOA(fVolumeFeedback[1])" 
	}
      }
    }
  RELEASE: {}
  }





BUTTON_EVENT[vdvTP____2,011]                                                   // Speaker 1:  Vol +
BUTTON_EVENT[vdvTP____2,021]                                                   // Speaker 2:  Vol +
BUTTON_EVENT[vdvTP____2,031]                                                   // Speaker 3:  Vol +
BUTTON_EVENT[vdvTP____2,041]                                                   // Speaker 4:  Vol +
BUTTON_EVENT[vdvTP____2,051]                                                   // Speaker 5:  Vol +
BUTTON_EVENT[vdvTP____2,061]                                                   // Speaker 6:  Vol +
BUTTON_EVENT[vdvTP____2,071]                                                   // Speaker 7:  Vol +
BUTTON_EVENT[vdvTP____2,081]                                                   // Speaker 8:  Vol +
  {
  PUSH:
    {
    STACK_VAR INTEGER nVdvSpkr_all_Index
    nVdvSpkr_all_Index = BUTTON.INPUT.CHANNEL/10                               // 1..8
    
    TO[BUTTON.INPUT]
    
    SWITCH(nLink)
      {
      CASE 0:  
	{ 
	SET_PULSE_TIME(2)
	PULSE[vdvSpkr_all[nVdvSpkr_all_Index],VOL_UP]                          // VOL_UP = 24    // Ramping:   Ramp volume up
	}
      CASE 1:  
	{ 
	SET_PULSE_TIME(2)
	PULSE[vdvSpkr_all,VOL_UP]                                              // VOL_UP = 24    // Ramping:   Ramp volume up
	}
      }
    }
  HOLD[10, REPEAT]:
    {
    STACK_VAR INTEGER nVdvSpkr_all_Index
    nVdvSpkr_all_Index = BUTTON.INPUT.CHANNEL/10                               // 1..8
    
    SWITCH(nLink)
      {
      CASE 0:  
	{ 
	SET_PULSE_TIME(2)
	PULSE[vdvSpkr_all[nVdvSpkr_all_Index],GAIN_UP]                         // GAIN_UP = 140   // Ramping:   Ramp gain up
	}
      CASE 1:  
	{ 
	SET_PULSE_TIME(2)
	PULSE[vdvSpkr_all,GAIN_UP]                                             // GAIN_UP = 140   // Ramping:   Ramp gain up
	}
      }
    }
  RELEASE:
    {
    }
  }
  
  
  
BUTTON_EVENT[vdvTP____2,012]                                                   // Speaker 1:  Vol -
BUTTON_EVENT[vdvTP____2,022]                                                   // Speaker 2:  Vol -
BUTTON_EVENT[vdvTP____2,032]                                                   // Speaker 3:  Vol -
BUTTON_EVENT[vdvTP____2,042]                                                   // Speaker 4:  Vol -
BUTTON_EVENT[vdvTP____2,052]                                                   // Speaker 5:  Vol -
BUTTON_EVENT[vdvTP____2,062]                                                   // Speaker 6:  Vol -
BUTTON_EVENT[vdvTP____2,072]                                                   // Speaker 7:  Vol -
BUTTON_EVENT[vdvTP____2,082]                                                   // Speaker 8:  Vol -
  {
  PUSH:
    {
    LOCAL_VAR INTEGER nVdvSpkr_all_Index
    nVdvSpkr_all_Index = BUTTON.INPUT.CHANNEL/10                               // 1..8
    
    TO[BUTTON.INPUT]
    
    SWITCH(nLink)
      {
      CASE 0:  
	{ 
	SET_PULSE_TIME(2)
	PULSE[vdvSpkr_all[nVdvSpkr_all_Index],VOL_DN]                          // VOL_DN = 25    // Ramping:   Ramp volume down
	}
      CASE 1:  
	{ 
	SET_PULSE_TIME(2)
	PULSE[vdvSpkr_all,VOL_DN]                                              // VOL_DN = 25    // Ramping:   Ramp volume down
	}
      }
    }
  HOLD[10, REPEAT]:
    {
    LOCAL_VAR INTEGER nVdvSpkr_all_Index
    nVdvSpkr_all_Index = BUTTON.INPUT.CHANNEL/10                               // 1..8
    
    SWITCH(nLink)
      {
      CASE 0:  
	{ 
	SET_PULSE_TIME(2)
	PULSE[vdvSpkr_all[nVdvSpkr_all_Index],GAIN_DN]                         // GAIN_DN = 141   // Ramping:   Ramp gain down
	}
      CASE 1:  
	{ 
	SET_PULSE_TIME(2)
	PULSE[vdvSpkr_all,GAIN_DN]                                             // GAIN_DN = 141   // Ramping:   Ramp gain down
	}
      }
    }
  RELEASE:
    {
    }
  }

  
  
  
BUTTON_EVENT[vdvTP____2,013]                                                   // Speaker 1:  Mute Toggle
BUTTON_EVENT[vdvTP____2,023]                                                   // Speaker 2:  Mute Toggle
BUTTON_EVENT[vdvTP____2,033]                                                   // Speaker 3:  Mute Toggle
BUTTON_EVENT[vdvTP____2,043]                                                   // Speaker 4:  Mute Toggle
BUTTON_EVENT[vdvTP____2,053]                                                   // Speaker 5:  Mute Toggle
BUTTON_EVENT[vdvTP____2,063]                                                   // Speaker 6:  Mute Toggle
BUTTON_EVENT[vdvTP____2,073]                                                   // Speaker 7:  Mute Toggle
BUTTON_EVENT[vdvTP____2,083]                                                   // Speaker 8:  Mute Toggle
  {
  PUSH:
    {
    LOCAL_VAR INTEGER nVdvSpkr_all_Index
    nVdvSpkr_all_Index = BUTTON.INPUT.CHANNEL/10                               // 1..8

    SWITCH(nLink)
      {
      CASE 1:
	{
	SWITCH(nVOL_MUTE_FB[1])
	  {
	  CASE 0:  { SEND_COMMAND vdvSpkr_all, "'VOL_MUTE-ON'"  }
	  DEFAULT: { SEND_COMMAND vdvSpkr_all, "'VOL_MUTE-OFF'" }
	  }
	}
      CASE 0:
	{
	SWITCH(nVOL_MUTE_FB[nVdvSpkr_all_Index])
	  {
	  CASE 0:  { SEND_COMMAND vdvSpkr_all[nVdvSpkr_all_Index], "'VOL_MUTE-ON'"  }
	  DEFAULT: { SEND_COMMAND vdvSpkr_all[nVdvSpkr_all_Index], "'VOL_MUTE-OFF'" }
	  }
	}
      }
    }
  RELEASE: {}
  }

  
  
  
BUTTON_EVENT[vdvTP____2,014]                                                   // Speaker 1:  set to -50dB 
BUTTON_EVENT[vdvTP____2,024]                                                   // Speaker 2:  set to -50dB   
BUTTON_EVENT[vdvTP____2,034]                                                   // Speaker 3:  set to -50dB   
BUTTON_EVENT[vdvTP____2,044]                                                   // Speaker 4:  set to -50dB   
BUTTON_EVENT[vdvTP____2,054]                                                   // Speaker 5:  set to -50dB   
BUTTON_EVENT[vdvTP____2,064]                                                   // Speaker 6:  set to -50dB   
BUTTON_EVENT[vdvTP____2,074]                                                   // Speaker 7:  set to -50dB   
BUTTON_EVENT[vdvTP____2,084]                                                   // Speaker 8:  set to -50dB   
  {
  PUSH:
    {
    LOCAL_VAR INTEGER nVdvSpkr_all_Index
    nVdvSpkr_all_Index = BUTTON.INPUT.CHANNEL/10                               // 1..8

    TO[BUTTON.INPUT]

    SWITCH(nLink)
      {
      CASE 1:
	{
	SEND_COMMAND vdvSpkr_all, "'VOL_LVL--50'"
	}
      CASE 0:
	{
	SEND_COMMAND vdvSpkr_all[nVdvSpkr_all_Index], "'VOL_LVL--50'"
	}
      }
    }
  RELEASE: {}
  }



  
  
  
BUTTON_EVENT[vdvTP____2,015]                                                   // Speaker 1:  Toggle for Analog input 
BUTTON_EVENT[vdvTP____2,025]                                                   // Speaker 2:  Toggle for Analog input   
BUTTON_EVENT[vdvTP____2,035]                                                   // Speaker 3:  Toggle for Analog input   
BUTTON_EVENT[vdvTP____2,045]                                                   // Speaker 4:  Toggle for Analog input   
BUTTON_EVENT[vdvTP____2,055]                                                   // Speaker 5:  Toggle for Analog input  
BUTTON_EVENT[vdvTP____2,065]                                                   // Speaker 6:  Toggle for Analog input   
BUTTON_EVENT[vdvTP____2,075]                                                   // Speaker 7:  Toggle for Analog input   
BUTTON_EVENT[vdvTP____2,085]                                                   // Speaker 8:  Toggle for Analog input  
  {
  PUSH:
    {
    LOCAL_VAR INTEGER nVdvSpkr_all_Index
    nVdvSpkr_all_Index = BUTTON.INPUT.CHANNEL/10                               // 1..8
    TO[vdvSpkr_all[nVdvSpkr_all_Index],030]                                    // Channel 30  = SOURCE_TV1      on/off Toggle for Analog input
    }
  RELEASE: {}
  }

  
BUTTON_EVENT[vdvTP____2,016]                                                   // Speaker 1:  Toggle for AoIP input channel 1  
BUTTON_EVENT[vdvTP____2,026]                                                   // Speaker 2:  Toggle for AoIP input channel 1    
BUTTON_EVENT[vdvTP____2,036]                                                   // Speaker 3:  Toggle for AoIP input channel 1    
BUTTON_EVENT[vdvTP____2,046]                                                   // Speaker 4:  Toggle for AoIP input channel 1    
BUTTON_EVENT[vdvTP____2,056]                                                   // Speaker 5:  Toggle for AoIP input channel 1   
BUTTON_EVENT[vdvTP____2,066]                                                   // Speaker 6:  Toggle for AoIP input channel 1    
BUTTON_EVENT[vdvTP____2,076]                                                   // Speaker 7:  Toggle for AoIP input channel 1    
BUTTON_EVENT[vdvTP____2,086]                                                   // Speaker 8:  Toggle for AoIP input channel 1   
  {
  PUSH:
    {
    LOCAL_VAR INTEGER nVdvSpkr_all_Index
    nVdvSpkr_all_Index = BUTTON.INPUT.CHANNEL/10                               // 1..8
    TO[vdvSpkr_all[nVdvSpkr_all_Index],031]                                    // Channel 31  = SOURCE_VIDEO1   on/off Toggle for AoIP input channel 1  
    }
  RELEASE: {}
  }


  
BUTTON_EVENT[vdvTP____2,017]                                                   // Speaker 1:  Toggle for AoIP input channel 2
BUTTON_EVENT[vdvTP____2,027]                                                   // Speaker 2:  Toggle for AoIP input channel 2   
BUTTON_EVENT[vdvTP____2,037]                                                   // Speaker 3:  Toggle for AoIP input channel 2   
BUTTON_EVENT[vdvTP____2,047]                                                   // Speaker 4:  Toggle for AoIP input channel 2   
BUTTON_EVENT[vdvTP____2,057]                                                   // Speaker 5:  Toggle for AoIP input channel 2  
BUTTON_EVENT[vdvTP____2,067]                                                   // Speaker 6:  Toggle for AoIP input channel 2   
BUTTON_EVENT[vdvTP____2,077]                                                   // Speaker 7:  Toggle for AoIP input channel 2   
BUTTON_EVENT[vdvTP____2,087]                                                   // Speaker 8:  Toggle for AoIP input channel 2  
  {
  PUSH:
    {
    LOCAL_VAR INTEGER nVdvSpkr_all_Index
    nVdvSpkr_all_Index = BUTTON.INPUT.CHANNEL/10                               // 1..8
    TO[vdvSpkr_all[nVdvSpkr_all_Index],032]                                    // Channel 32  = SOURCE_VIDEO2   on/off Toggle for AoIP input channel 2
    }
  RELEASE: {}
  }






BUTTON_EVENT[vdvTP____2,100]                                                   // Profile 0
BUTTON_EVENT[vdvTP____2,101]                                                   // Profile 1
BUTTON_EVENT[vdvTP____2,102]                                                   // Profile 2
BUTTON_EVENT[vdvTP____2,103]                                                   // Profile 3
BUTTON_EVENT[vdvTP____2,104]                                                   // Profile 4
BUTTON_EVENT[vdvTP____2,105]                                                   // Profile 5
  {
  PUSH:
    {
    TO[BUTTON.INPUT]
    SEND_COMMAND vdvSpkr_all, "'PRESET-',ITOA(BUTTON.INPUT.CHANNEL-100)" 
    }
  RELEASE: {}
  }









BUTTON_EVENT[vdvTP____2,221]                                                   // Speaker 1: show info popup
BUTTON_EVENT[vdvTP____2,222]                                                   // Speaker 2: show info popup
BUTTON_EVENT[vdvTP____2,223]                                                   // Speaker 3: show info popup
BUTTON_EVENT[vdvTP____2,224]                                                   // Speaker 4: show info popup
BUTTON_EVENT[vdvTP____2,225]                                                   // Speaker 5: show info popup
BUTTON_EVENT[vdvTP____2,226]                                                   // Speaker 6: show info popup
BUTTON_EVENT[vdvTP____2,227]                                                   // Speaker 7: show info popup
BUTTON_EVENT[vdvTP____2,228]                                                   // Speaker 8: show info popup
  {
  PUSH:
    {
    SEND_COMMAND vdvSpkr_all[BUTTON.INPUT.CHANNEL-220], "'GET_DEVICE_INFO-/device/pwr'" 
    SEND_COMMAND vdvSpkr_all[BUTTON.INPUT.CHANNEL-220], "'GET_DEVICE_INFO-/audio/inputs'" 
    SEND_COMMAND vdvSpkr_all[BUTTON.INPUT.CHANNEL-220], "'GET_DEVICE_INFO-/audio/volume'" 
    SEND_COMMAND vdvSpkr_all[BUTTON.INPUT.CHANNEL-220], "'GET_DEVICE_INFO-/events'" 
    SEND_COMMAND vdvSpkr_all[BUTTON.INPUT.CHANNEL-220], "'GET_DEVICE_INFO-/network/zone'" 
    SEND_COMMAND vdvSpkr_all[BUTTON.INPUT.CHANNEL-220], "'GET_DEVICE_INFO-/profile/list'" 
    SEND_COMMAND vdvSpkr_all[BUTTON.INPUT.CHANNEL-220], "'GET_DEVICE_INFO-/device/id'" 
    SEND_COMMAND vdvSpkr_all[BUTTON.INPUT.CHANNEL-220], "'GET_DEVICE_INFO-/device/info'" 
    }
  RELEASE: {}
  }


BUTTON_EVENT[vdvTP____2,229]                                                   // exit info popup
  {
  PUSH:
    {
    ClearInfoPopup()
    }
  RELEASE: {}
  }





(************************************************************************)
(* Start  TIMELINE_EVENT  TL2_ID_FEEDBACK                               *)
(************************************************************************)
TIMELINE_EVENT[TL2_ID_FEEDBACK]                                                // Timeline-2: Feedback  200ms
  {

  [vdvTP____2,009] =  nLink
  
  [vdvTP____2,013] = (nVOL_MUTE_FB[1] = 1)
  [vdvTP____2,023] = (nVOL_MUTE_FB[2] = 1)
//[vdvTP____2,033] = (nVOL_MUTE_FB[3] = 1)
//[vdvTP____2,043] = (nVOL_MUTE_FB[4] = 1)
//[vdvTP____2,053] = (nVOL_MUTE_FB[5] = 1)
//[vdvTP____2,063] = (nVOL_MUTE_FB[6] = 1)
//[vdvTP____2,073] = (nVOL_MUTE_FB[7] = 1)
//[vdvTP____2,083] = (nVOL_MUTE_FB[8] = 1)

// Feedback for Analog input
  [vdvTP____2,015] = (nSELECTED_INPUT_FB[1][1] = 1)  
  [vdvTP____2,025] = (nSELECTED_INPUT_FB[2][1] = 1)
//[vdvTP____2,035] = (nSELECTED_INPUT_FB[3][1] = 1)
//[vdvTP____2,045] = (nSELECTED_INPUT_FB[4][1] = 1)
//[vdvTP____2,055] = (nSELECTED_INPUT_FB[5][1] = 1)
//[vdvTP____2,065] = (nSELECTED_INPUT_FB[6][1] = 1)
//[vdvTP____2,075] = (nSELECTED_INPUT_FB[7][1] = 1)
//[vdvTP____2,085] = (nSELECTED_INPUT_FB[8][1] = 1)

// Feedback for AoIP input channel 1
  [vdvTP____2,016] = (nSELECTED_INPUT_FB[1][2] = 1)  
  [vdvTP____2,026] = (nSELECTED_INPUT_FB[2][2] = 1)
//[vdvTP____2,036] = (nSELECTED_INPUT_FB[3][2] = 1)
//[vdvTP____2,046] = (nSELECTED_INPUT_FB[4][2] = 1)
//[vdvTP____2,056] = (nSELECTED_INPUT_FB[5][2] = 1)
//[vdvTP____2,066] = (nSELECTED_INPUT_FB[6][2] = 1)
//[vdvTP____2,076] = (nSELECTED_INPUT_FB[7][2] = 1)
//[vdvTP____2,086] = (nSELECTED_INPUT_FB[8][2] = 1)

// Feedback for AoIP input channel 2
  [vdvTP____2,017] = (nSELECTED_INPUT_FB[1][3] = 1)  
  [vdvTP____2,027] = (nSELECTED_INPUT_FB[2][3] = 1)
//[vdvTP____2,037] = (nSELECTED_INPUT_FB[3][3] = 1)
//[vdvTP____2,047] = (nSELECTED_INPUT_FB[4][3] = 1)
//[vdvTP____2,057] = (nSELECTED_INPUT_FB[5][3] = 1)
//[vdvTP____2,067] = (nSELECTED_INPUT_FB[6][3] = 1)
//[vdvTP____2,077] = (nSELECTED_INPUT_FB[7][3] = 1)
//[vdvTP____2,087] = (nSELECTED_INPUT_FB[8][3] = 1)

  [vdvTP____2,231] = (nPOWER_FB[1] = 1)
  [vdvTP____2,232] = (nPOWER_FB[2] = 1)
//[vdvTP____2,233] = (nPOWER_FB[3] = 1)
//[vdvTP____2,234] = (nPOWER_FB[4] = 1)
//[vdvTP____2,235] = (nPOWER_FB[5] = 1)
//[vdvTP____2,236] = (nPOWER_FB[6] = 1)
//[vdvTP____2,237] = (nPOWER_FB[7] = 1)
//[vdvTP____2,238] = (nPOWER_FB[8] = 1)

  [vdvTP____2,241] = (nDEVICE_COMMUNICATING[1] = 1)
  [vdvTP____2,242] = (nDEVICE_COMMUNICATING[2] = 1)
//[vdvTP____2,243] = (nDEVICE_COMMUNICATING[3] = 1)
//[vdvTP____2,244] = (nDEVICE_COMMUNICATING[4] = 1)
//[vdvTP____2,245] = (nDEVICE_COMMUNICATING[5] = 1)
//[vdvTP____2,246] = (nDEVICE_COMMUNICATING[6] = 1)
//[vdvTP____2,247] = (nDEVICE_COMMUNICATING[7] = 1)
//[vdvTP____2,248] = (nDEVICE_COMMUNICATING[8] = 1)

// Feedback selected profile
  [vdvTP____2,100] = (nProfileSelected = 0)
  [vdvTP____2,101] = (nProfileSelected = 1)
  [vdvTP____2,102] = (nProfileSelected = 2)
  [vdvTP____2,103] = (nProfileSelected = 3)
  [vdvTP____2,104] = (nProfileSelected = 4)
  [vdvTP____2,105] = (nProfileSelected = 5)



  }//end TIMELINE_EVENT
(************************************************************************)
(* End    TIMELINE_EVENT  TL2_ID_FEEDBACK                               *)
(************************************************************************)



